from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..pagination import PaginatedResponse
from ..types import DataSource, DataSourceVersion
from .base import AsyncBaseResource, BaseResource


class DataSourcesResource(BaseResource):
    def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[DataSource]:
        body = self._http.get("/api/v1/data-sources", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, DataSource.from_dict)

    def get(self, data_source_id: str) -> DataSource:
        body = self._http.get(f"/api/v1/data-sources/{data_source_id}")
        return DataSource.from_dict(self._unwrap(body))

    def create(self, *, name: str, type: str) -> DataSource:
        body = self._http.post("/api/v1/data-sources", json={"name": name, "type": type})
        return DataSource.from_dict(self._unwrap(body))

    def update(self, data_source_id: str, **kwargs) -> DataSource:
        body = self._http.patch(f"/api/v1/data-sources/{data_source_id}", json=kwargs)
        return DataSource.from_dict(self._unwrap(body))

    def delete(self, data_source_id: str) -> None:
        self._http.delete(f"/api/v1/data-sources/{data_source_id}")

    def list_versions(self, data_source_id: str) -> List[DataSourceVersion]:
        body = self._http.get(f"/api/v1/data-sources/{data_source_id}/versions")
        data = self._unwrap(body)
        return [DataSourceVersion.from_dict(v) for v in (data if isinstance(data, list) else [])]

    def create_version(
        self,
        data_source_id: str,
        *,
        version: str,
        query_template: str = "",
        credential_id: Optional[str] = None,
        connection_config: Optional[Dict[str, Any]] = None,
        parameters: Optional[List[Any]] = None,
        set_current: bool = False,
        message: Optional[str] = None,
    ) -> DataSourceVersion:
        payload: Dict[str, Any] = {
            "version": version,
            "query_template": query_template,
            "set_current": set_current,
        }
        if credential_id is not None:
            payload["credential_id"] = credential_id
        if connection_config is not None:
            payload["connection_config"] = connection_config
        if parameters is not None:
            payload["parameters"] = parameters
        if message is not None:
            payload["message"] = message
        body = self._http.post(f"/api/v1/data-sources/{data_source_id}/versions", json=payload)
        return DataSourceVersion.from_dict(self._unwrap(body))

    def promote_version(self, data_source_id: str, version_id: str) -> Dict[str, Any]:
        body = self._http.put(
            f"/api/v1/data-sources/{data_source_id}/versions/{version_id}/promote", json={}
        )
        return self._unwrap(body)

    def preview_query(
        self,
        data_source_id: str,
        *,
        query_template: str,
        parameters: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {"query_template": query_template}
        if parameters is not None:
            payload["parameters"] = parameters
        body = self._http.post(f"/api/v1/data-sources/{data_source_id}/preview-query", json=payload)
        return self._unwrap(body)

    def test_connection(self, data_source_id: str) -> Dict[str, Any]:
        body = self._http.post(f"/api/v1/data-sources/{data_source_id}/test-connection")
        return self._unwrap(body)

    def query(
        self, data_source_id: str, *, parameters: Optional[Dict[str, Any]] = None
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {}
        if parameters is not None:
            payload["parameters"] = parameters
        body = self._http.post(f"/api/v1/data-sources/{data_source_id}/query", json=payload)
        return self._unwrap(body)


class AsyncDataSourcesResource(AsyncBaseResource):
    async def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[DataSource]:
        body = await self._http.get("/api/v1/data-sources", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, DataSource.from_dict)

    async def get(self, data_source_id: str) -> DataSource:
        body = await self._http.get(f"/api/v1/data-sources/{data_source_id}")
        return DataSource.from_dict(self._unwrap(body))

    async def create(self, *, name: str, type: str) -> DataSource:
        body = await self._http.post("/api/v1/data-sources", json={"name": name, "type": type})
        return DataSource.from_dict(self._unwrap(body))

    async def update(self, data_source_id: str, **kwargs) -> DataSource:
        body = await self._http.patch(f"/api/v1/data-sources/{data_source_id}", json=kwargs)
        return DataSource.from_dict(self._unwrap(body))

    async def delete(self, data_source_id: str) -> None:
        await self._http.delete(f"/api/v1/data-sources/{data_source_id}")

    async def list_versions(self, data_source_id: str) -> List[DataSourceVersion]:
        body = await self._http.get(f"/api/v1/data-sources/{data_source_id}/versions")
        data = self._unwrap(body)
        return [DataSourceVersion.from_dict(v) for v in (data if isinstance(data, list) else [])]

    async def create_version(
        self,
        data_source_id: str,
        *,
        version: str,
        query_template: str = "",
        credential_id: Optional[str] = None,
        connection_config: Optional[Dict[str, Any]] = None,
        parameters: Optional[List[Any]] = None,
        set_current: bool = False,
        message: Optional[str] = None,
    ) -> DataSourceVersion:
        payload: Dict[str, Any] = {
            "version": version,
            "query_template": query_template,
            "set_current": set_current,
        }
        if credential_id is not None:
            payload["credential_id"] = credential_id
        if connection_config is not None:
            payload["connection_config"] = connection_config
        if parameters is not None:
            payload["parameters"] = parameters
        if message is not None:
            payload["message"] = message
        body = await self._http.post(
            f"/api/v1/data-sources/{data_source_id}/versions", json=payload
        )
        return DataSourceVersion.from_dict(self._unwrap(body))

    async def promote_version(self, data_source_id: str, version_id: str) -> Dict[str, Any]:
        body = await self._http.put(
            f"/api/v1/data-sources/{data_source_id}/versions/{version_id}/promote", json={}
        )
        return self._unwrap(body)

    async def preview_query(
        self,
        data_source_id: str,
        *,
        query_template: str,
        parameters: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {"query_template": query_template}
        if parameters is not None:
            payload["parameters"] = parameters
        body = await self._http.post(
            f"/api/v1/data-sources/{data_source_id}/preview-query", json=payload
        )
        return self._unwrap(body)

    async def test_connection(self, data_source_id: str) -> Dict[str, Any]:
        body = await self._http.post(f"/api/v1/data-sources/{data_source_id}/test-connection")
        return self._unwrap(body)

    async def query(
        self, data_source_id: str, *, parameters: Optional[Dict[str, Any]] = None
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {}
        if parameters is not None:
            payload["parameters"] = parameters
        body = await self._http.post(f"/api/v1/data-sources/{data_source_id}/query", json=payload)
        return self._unwrap(body)



from __future__ import annotations

from ._http import HTTPClient
from .config import Config
from .resources import (
    A2AResource,
    AgentsResource,
    ApprovalsResource,
    AssetsResource,
    ChatResource,
    CostsResource,
    CredentialsResource,
    DashboardResource,
    DataSourcesResource,
    ExecutionsResource,
    GuardrailsResource,
    LLMModelsResource,
    MCPTemplatesResource,
    MCPToolsResource,
    MediaModelsResource,
    MediaResource,
    MemoriesResource,
    PromptsResource,
    ScoresResource,
    SessionsResource,
    TemplatesResource,
    TracesResource,
    WebhookTriggersResource,
)


class PromptRails:
    """Synchronous PromptRails API client."""

    def __init__(
        self,
        api_key: str,
        *,
        base_url: str = "https://api.promptrails.ai",
        timeout: float = 30.0,
        max_retries: int = 3,
    ):
        self._config = Config(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
        )
        self._http = HTTPClient(self._config)

        self.agents = AgentsResource(self._http)
        self.prompts = PromptsResource(self._http)
        self.executions = ExecutionsResource(self._http)
        self.credentials = CredentialsResource(self._http)
        self.data_sources = DataSourcesResource(self._http)
        self.chat = ChatResource(self._http)
        self.memories = MemoriesResource(self._http)
        self.traces = TracesResource(self._http)
        self.costs = CostsResource(self._http)
        self.mcp_tools = MCPToolsResource(self._http)
        self.mcp_templates = MCPTemplatesResource(self._http)
        self.guardrails = GuardrailsResource(self._http)
        self.llm_models = LLMModelsResource(self._http)
        self.approvals = ApprovalsResource(self._http)
        self.templates = TemplatesResource(self._http)
        self.scores = ScoresResource(self._http)
        self.dashboard = DashboardResource(self._http)
        self.sessions = SessionsResource(self._http)
        self.a2a = A2AResource(self._http)
        self.webhook_triggers = WebhookTriggersResource(self._http)
        self.media_models = MediaModelsResource(self._http)
        self.media = MediaResource(self._http)
        self.assets = AssetsResource(self._http)

    def close(self):
        """Close the underlying HTTP client."""
        self._http.close()

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import Asset, AssetSignedURL
from .base import AsyncBaseResource, BaseResource


class AssetsResource(BaseResource):
    """Resource for managing media assets."""

    def list(
        self,
        *,
        type: Optional[str] = None,
        provider: Optional[str] = None,
        execution_id: Optional[str] = None,
        agent_id: Optional[str] = None,
        page: int = 1,
        limit: int = 20,
    ) -> PaginatedResponse[Asset]:
        """List assets.

        Args:
            type: Filter by asset type (e.g. ``image``, ``speech``, ``video``).
            provider: Filter by provider.
            execution_id: Filter by execution ID.
            agent_id: Filter by agent ID.
            page: Page number.
            limit: Items per page.

        Returns:
            Paginated list of assets.
        """
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if type is not None:
            params["type"] = type
        if provider is not None:
            params["provider"] = provider
        if execution_id is not None:
            params["execution_id"] = execution_id
        if agent_id is not None:
            params["agent_id"] = agent_id
        body = self._http.get("/api/v1/assets", params=params)
        return PaginatedResponse.from_response(body, Asset.from_dict)

    def get(self, asset_id: str) -> Asset:
        """Get a single asset by ID.

        Args:
            asset_id: The asset ID.

        Returns:
            The asset.
        """
        body = self._http.get(f"/api/v1/assets/{asset_id}")
        return Asset.from_dict(self._unwrap(body))

    def delete(self, asset_id: str) -> None:
        """Delete an asset.

        Args:
            asset_id: The asset ID.
        """
        self._http.delete(f"/api/v1/assets/{asset_id}")

    def get_signed_url(self, asset_id: str) -> AssetSignedURL:
        """Get a signed URL for an asset.

        Args:
            asset_id: The asset ID.

        Returns:
            Signed URL details.
        """
        body = self._http.get(f"/api/v1/assets/{asset_id}/signed-url")
        return AssetSignedURL.from_dict(self._unwrap(body))


class AsyncAssetsResource(AsyncBaseResource):
    """Async resource for managing media assets."""

    async def list(
        self,
        *,
        type: Optional[str] = None,
        provider: Optional[str] = None,
        execution_id: Optional[str] = None,
        agent_id: Optional[str] = None,
        page: int = 1,
        limit: int = 20,
    ) -> PaginatedResponse[Asset]:
        """List assets.

        Args:
            type: Filter by asset type (e.g. ``image``, ``speech``, ``video``).
            provider: Filter by provider.
            execution_id: Filter by execution ID.
            agent_id: Filter by agent ID.
            page: Page number.
            limit: Items per page.

        Returns:
            Paginated list of assets.
        """
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if type is not None:
            params["type"] = type
        if provider is not None:
            params["provider"] = provider
        if execution_id is not None:
            params["execution_id"] = execution_id
        if agent_id is not None:
            params["agent_id"] = agent_id
        body = await self._http.get("/api/v1/assets", params=params)
        return PaginatedResponse.from_response(body, Asset.from_dict)

    async def get(self, asset_id: str) -> Asset:
        """Get a single asset by ID.

        Args:
            asset_id: The asset ID.

        Returns:
            The asset.
        """
        body = await self._http.get(f"/api/v1/assets/{asset_id}")
        return Asset.from_dict(self._unwrap(body))

    async def delete(self, asset_id: str) -> None:
        """Delete an asset.

        Args:
            asset_id: The asset ID.
        """
        await self._http.delete(f"/api/v1/assets/{asset_id}")

    async def get_signed_url(self, asset_id: str) -> AssetSignedURL:
        """Get a signed URL for an asset.

        Args:
            asset_id: The asset ID.

        Returns:
            Signed URL details.
        """
        body = await self._http.get(f"/api/v1/assets/{asset_id}/signed-url")
        return AssetSignedURL.from_dict(self._unwrap(body))



from __future__ import annotations

from typing import Any, Dict, Optional

from ..types import CostSummary
from .base import AsyncBaseResource, BaseResource


class CostsResource(BaseResource):
    def get_summary(
        self, *, from_date: Optional[str] = None, to_date: Optional[str] = None
    ) -> CostSummary:
        params: Dict[str, Any] = {}
        if from_date:
            params["from"] = from_date
        if to_date:
            params["to"] = to_date
        body = self._http.get("/api/v1/costs/summary", params=params)
        return CostSummary.from_dict(self._unwrap(body))

    def get_agent_summary(
        self, agent_id: str, *, from_date: Optional[str] = None, to_date: Optional[str] = None
    ) -> CostSummary:
        params: Dict[str, Any] = {}
        if from_date:
            params["from"] = from_date
        if to_date:
            params["to"] = to_date
        body = self._http.get(f"/api/v1/costs/agents/{agent_id}", params=params)
        return CostSummary.from_dict(self._unwrap(body))


class AsyncCostsResource(AsyncBaseResource):
    async def get_summary(
        self, *, from_date: Optional[str] = None, to_date: Optional[str] = None
    ) -> CostSummary:
        params: Dict[str, Any] = {}
        if from_date:
            params["from"] = from_date
        if to_date:
            params["to"] = to_date
        body = await self._http.get("/api/v1/costs/summary", params=params)
        return CostSummary.from_dict(self._unwrap(body))

    async def get_agent_summary(
        self, agent_id: str, *, from_date: Optional[str] = None, to_date: Optional[str] = None
    ) -> CostSummary:
        params: Dict[str, Any] = {}
        if from_date:
            params["from"] = from_date
        if to_date:
            params["to"] = to_date
        body = await self._http.get(f"/api/v1/costs/agents/{agent_id}", params=params)
        return CostSummary.from_dict(self._unwrap(body))



from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional


def _from_dict(cls, data: Dict[str, Any]):
    """Create a dataclass instance from a dict, ignoring unknown keys."""
    if data is None:
        return None
    known = {f.name for f in cls.__dataclass_fields__.values()}
    return cls(**{k: v for k, v in data.items() if k in known})


# --- Agent ---


@dataclass
class AgentVersion:
    id: str = ""
    agent_id: str = ""
    version: str = ""
    config: Dict[str, Any] = field(default_factory=dict)
    input_schema: Dict[str, Any] = field(default_factory=dict)
    output_schema: Optional[Dict[str, Any]] = None
    is_current: bool = False
    message: str = ""
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AgentVersion":
        return _from_dict(cls, data)


@dataclass
class Agent:
    id: str = ""
    workspace_id: str = ""
    name: str = ""
    description: str = ""
    type: str = ""
    status: str = "draft"
    labels: List[str] = field(default_factory=list)
    created_by_id: Optional[str] = None
    created_at: str = ""
    updated_at: str = ""
    current_version: Optional[AgentVersion] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Agent":
        obj = _from_dict(cls, data)
        if data.get("current_version"):
            obj.current_version = AgentVersion.from_dict(data["current_version"])
        return obj


# --- Prompt ---


@dataclass
class LLMModel:
    id: str = ""
    provider: str = ""
    model_id: str = ""
    display_name: str = ""
    input_price: Optional[float] = None
    output_price: Optional[float] = None
    max_tokens: Optional[int] = None
    supports_vision: bool = False
    supports_tools: bool = False
    supports_json: bool = False
    supports_streaming: bool = False
    is_active: bool = True

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "LLMModel":
        return _from_dict(cls, data)


@dataclass
class RunPromptResponse:
    content: str = ""
    token_usage: Dict[str, Any] = field(default_factory=dict)
    cost: float = 0.0
    duration_ms: int = 0
    model: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "RunPromptResponse":
        return _from_dict(cls, data)


@dataclass
class AvailableModelEntry:
    id: str = ""
    model_id: str = ""
    display_name: str = ""
    max_tokens: Optional[int] = None
    supports_vision: bool = False
    supports_tools: bool = False
    supports_json: bool = False
    input_price: Optional[float] = None
    output_price: Optional[float] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AvailableModelEntry":
        return _from_dict(cls, data)


@dataclass
class AvailableModelGroup:
    provider: str = ""
    models: List[AvailableModelEntry] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AvailableModelGroup":
        obj = _from_dict(cls, data)
        if data.get("models"):
            obj.models = [AvailableModelEntry.from_dict(m) for m in data["models"]]
        return obj


@dataclass
class PromptVersion:
    id: str = ""
    prompt_id: str = ""
    version: str = ""
    system_prompt: str = ""
    user_prompt: str = ""
    llm_model_id: Optional[str] = None
    fallback_llm_model_id: Optional[str] = None
    temperature: Optional[float] = None
    max_tokens: Optional[int] = None
    top_p: Optional[float] = None
    input_schema: Dict[str, Any] = field(default_factory=dict)
    output_schema: Optional[Dict[str, Any]] = None
    is_current: bool = False
    config: Dict[str, Any] = field(default_factory=dict)
    message: str = ""
    created_at: str = ""
    llm_model: Optional[LLMModel] = None
    fallback_llm_model: Optional[LLMModel] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "PromptVersion":
        obj = _from_dict(cls, data)
        if data.get("llm_model"):
            obj.llm_model = LLMModel.from_dict(data["llm_model"])
        if data.get("fallback_llm_model"):
            obj.fallback_llm_model = LLMModel.from_dict(data["fallback_llm_model"])
        return obj


@dataclass
class Prompt:
    id: str = ""
    workspace_id: str = ""
    name: str = ""
    description: str = ""
    created_at: str = ""
    updated_at: str = ""
    current_version: Optional[PromptVersion] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Prompt":
        obj = _from_dict(cls, data)
        if data.get("current_version"):
            obj.current_version = PromptVersion.from_dict(data["current_version"])
        return obj


# --- Execution ---


@dataclass
class AgentExecution:
    id: str = ""
    agent_id: str = ""
    agent_version_id: str = ""
    workspace_id: str = ""
    user_id: str = ""
    session_id: str = ""
    status: str = ""
    input: Dict[str, Any] = field(default_factory=dict)
    output: Optional[Dict[str, Any]] = None
    error: str = ""
    metadata: Dict[str, Any] = field(default_factory=dict)
    token_usage: Dict[str, Any] = field(default_factory=dict)
    cost: float = 0.0
    duration_ms: Optional[int] = None
    trace_id: Optional[str] = None
    started_at: Optional[str] = None
    completed_at: Optional[str] = None
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AgentExecution":
        return _from_dict(cls, data)


# --- Credential ---


@dataclass
class Credential:
    id: str = ""
    workspace_id: str = ""
    name: str = ""
    type: str = ""
    category: str = ""
    description: str = ""
    masked_content: str = ""
    is_default: bool = False
    schema_type: str = ""
    is_valid: bool = True
    has_schema: bool = False
    schema_updated_at: Optional[str] = None
    created_by_id: Optional[str] = None
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Credential":
        return _from_dict(cls, data)


# --- DataSource ---


@dataclass
class DataSourceVersion:
    id: str = ""
    data_source_id: str = ""
    version: str = ""
    credential_id: Optional[str] = None
    connection_config: Dict[str, Any] = field(default_factory=dict)
    query_template: str = ""
    parameters: List[Any] = field(default_factory=list)
    is_current: bool = False
    message: str = ""
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DataSourceVersion":
        return _from_dict(cls, data)


@dataclass
class DataSource:
    id: str = ""
    workspace_id: str = ""
    name: str = ""
    type: str = ""
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DataSource":
        return _from_dict(cls, data)


# --- Chat ---


@dataclass
class ChatMessage:
    id: str = ""
    session_id: str = ""
    role: str = ""
    content: str = ""
    metadata: Dict[str, Any] = field(default_factory=dict)
    tool_calls: Optional[Dict[str, Any]] = None
    tool_results: Optional[Dict[str, Any]] = None
    model: Optional[str] = None
    cost: Optional[float] = None
    token_count: Optional[int] = None
    prompt_tokens: Optional[int] = None
    completion_tokens: Optional[int] = None
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ChatMessage":
        return _from_dict(cls, data)


@dataclass
class ChatSession:
    id: str = ""
    workspace_id: str = ""
    agent_id: str = ""
    user_id: str = ""
    title: str = ""
    metadata: Dict[str, Any] = field(default_factory=dict)
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ChatSession":
        return _from_dict(cls, data)


# --- Memory ---


@dataclass
class AgentMemory:
    id: str = ""
    workspace_id: str = ""
    agent_id: str = ""
    content: str = ""
    metadata: Optional[Dict[str, Any]] = None
    memory_type: str = ""
    importance: float = 0.5
    access_count: int = 0
    last_accessed_at: Optional[str] = None
    chat_session_id: Optional[str] = None
    created_by_id: Optional[str] = None
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AgentMemory":
        return _from_dict(cls, data)


# --- Trace ---


@dataclass
class Trace:
    id: str = ""
    workspace_id: str = ""
    trace_id: str = ""
    span_id: str = ""
    parent_span_id: str = ""
    name: str = ""
    kind: str = ""
    status: str = "ok"
    input: Optional[Dict[str, Any]] = None
    output: Optional[Dict[str, Any]] = None
    attributes: Dict[str, Any] = field(default_factory=dict)
    token_usage: Optional[Dict[str, Any]] = None
    cost: Optional[float] = None
    duration_ms: Optional[int] = None
    error_message: str = ""
    error_type: str = ""
    error_stack: str = ""
    prompt_tokens: Optional[int] = None
    completion_tokens: Optional[int] = None
    total_tokens: Optional[int] = None
    agent_id: Optional[str] = None
    prompt_id: Optional[str] = None
    llm_model_id: Optional[str] = None
    user_id: Optional[str] = None
    session_id: str = ""
    execution_id: Optional[str] = None
    service_name: str = ""
    started_at: str = ""
    ended_at: Optional[str] = None
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Trace":
        return _from_dict(cls, data)


# --- Cost ---


@dataclass
class CostSummary:
    total_cost: float = 0.0
    total_executions: int = 0
    total_tokens: int = 0
    daily_costs: List[Dict[str, Any]] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CostSummary":
        return _from_dict(cls, data)


# --- MCP Tool ---


@dataclass
class MCPTool:
    id: str = ""
    workspace_id: str = ""
    name: str = ""
    type: str = ""
    config: Dict[str, Any] = field(default_factory=dict)
    schema: Optional[Dict[str, Any]] = None
    is_active: bool = True
    credential_id: Optional[str] = None
    template_id: Optional[str] = None
    status: str = ""
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MCPTool":
        return _from_dict(cls, data)


@dataclass
class MCPDiscoveredTool:
    name: str = ""
    description: str = ""
    input_schema: Optional[Dict[str, Any]] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MCPDiscoveredTool":
        return cls(
            name=data.get("name", ""),
            description=data.get("description", ""),
            input_schema=data.get("inputSchema"),
        )


@dataclass
class MCPToolCallContent:
    type: str = ""
    text: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MCPToolCallContent":
        return cls(type=data.get("type", ""), text=data.get("text", ""))


@dataclass
class MCPDiscoverResult:
    tools: List["MCPDiscoveredTool"] = field(default_factory=list)
    trace_id: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MCPDiscoverResult":
        tools = [MCPDiscoveredTool.from_dict(t) for t in data.get("tools", [])]
        return cls(tools=tools, trace_id=data.get("trace_id", ""))


@dataclass
class MCPCallToolResult:
    content: List["MCPToolCallContent"] = field(default_factory=list)
    is_error: bool = False
    trace_id: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MCPCallToolResult":
        content = [MCPToolCallContent.from_dict(c) for c in data.get("content", [])]
        return cls(
            content=content,
            is_error=data.get("is_error", False),
            trace_id=data.get("trace_id", ""),
        )


# --- MCP Template ---


@dataclass
class MCPTemplateParameterSpec:
    name: str = ""
    type: str = ""
    required: bool = False
    secret: bool = False
    description: str = ""
    default: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MCPTemplateParameterSpec":
        return _from_dict(cls, data)


@dataclass
class MCPTemplate:
    id: str = ""
    slug: str = ""
    name: str = ""
    description: str = ""
    type: str = ""
    category: str = ""
    icon_url: str = ""
    config: Dict[str, Any] = field(default_factory=dict)
    required_parameters: List[MCPTemplateParameterSpec] = field(default_factory=list)
    documentation_url: str = ""
    setup_instructions: str = ""
    is_active: bool = True
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MCPTemplate":
        obj = _from_dict(cls, data)
        if data.get("required_parameters"):
            obj.required_parameters = [
                MCPTemplateParameterSpec.from_dict(p) for p in data["required_parameters"]
            ]
        return obj


@dataclass
class InstallMCPTemplateRequest:
    name: str = ""
    parameters: Dict[str, str] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "InstallMCPTemplateRequest":
        return _from_dict(cls, data)


# --- Guardrail ---


@dataclass
class Guardrail:
    id: str = ""
    agent_id: str = ""
    type: str = ""
    scanner_type: str = ""
    action: str = "block"
    config: Dict[str, Any] = field(default_factory=dict)
    is_active: bool = True
    sort_order: int = 0
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Guardrail":
        return _from_dict(cls, data)


# --- Approval ---


@dataclass
class ApprovalRequest:
    id: str = ""
    execution_id: str = ""
    workspace_id: str = ""
    agent_id: Optional[str] = None
    checkpoint_name: str = ""
    payload: Dict[str, Any] = field(default_factory=dict)
    status: str = "pending"
    reason: Optional[str] = None
    decided_by: Optional[str] = None
    decided_at: Optional[str] = None
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ApprovalRequest":
        return _from_dict(cls, data)


# --- Flow Template ---


@dataclass
class FlowTemplate:
    id: str = ""
    name: str = ""
    description: str = ""
    category: str = ""
    agent_type: str = ""
    complexity: str = ""
    tags: List[str] = field(default_factory=list)
    icon: str = ""
    node_count: int = 0
    flow_config: Dict[str, Any] = field(default_factory=dict)
    status: str = ""
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "FlowTemplate":
        return _from_dict(cls, data)


# --- Execution Result ---


@dataclass
class ExecutionResult:
    output: Optional[Dict[str, Any]] = None
    error: str = ""
    trace_id: str = ""
    execution_id: str = ""
    status: str = ""
    token_usage: Dict[str, Any] = field(default_factory=dict)
    cost: float = 0.0
    duration_ms: Optional[int] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ExecutionResult":
        return _from_dict(cls, data)


# --- Score ---


@dataclass
class Score:
    id: str = ""
    workspace_id: str = ""
    trace_id: str = ""
    span_id: str = ""
    name: str = ""
    value: Optional[float] = None
    string_value: str = ""
    data_type: str = "numeric"
    source: str = "manual"
    config_id: Optional[str] = None
    comment: str = ""
    created_by: str = ""
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Score":
        return _from_dict(cls, data)


@dataclass
class ScoreConfig:
    id: str = ""
    workspace_id: str = ""
    name: str = ""
    data_type: str = "numeric"
    min_value: Optional[float] = None
    max_value: Optional[float] = None
    categories: List[Dict[str, Any]] = field(default_factory=list)
    description: str = ""
    is_active: bool = True
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ScoreConfig":
        return _from_dict(cls, data)


@dataclass
class ScoreAggregate:
    name: str = ""
    count: int = 0
    avg: float = 0.0
    min: float = 0.0
    max: float = 0.0

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ScoreAggregate":
        return _from_dict(cls, data)


# --- Dashboard ---


@dataclass
class DashboardMetrics:
    overview: Dict[str, Any] = field(default_factory=dict)
    executions_by_day: List[Dict[str, Any]] = field(default_factory=list)
    cost_by_day: List[Dict[str, Any]] = field(default_factory=list)
    agent_usage: List[Dict[str, Any]] = field(default_factory=list)
    model_usage: List[Dict[str, Any]] = field(default_factory=list)
    error_rate: List[Dict[str, Any]] = field(default_factory=list)
    score_trends: List[Dict[str, Any]] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DashboardMetrics":
        return _from_dict(cls, data)


# --- Session & User ---


@dataclass
class SessionSummary:
    session_id: str = ""
    trace_count: int = 0
    total_tokens: int = 0
    total_cost: float = 0.0
    error_count: int = 0
    first_seen: str = ""
    last_seen: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "SessionSummary":
        return _from_dict(cls, data)


# --- A2A (Agent-to-Agent) ---


@dataclass
class A2APart:
    type: str = ""
    text: str = ""
    data: Optional[Dict[str, Any]] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2APart":
        return _from_dict(cls, data)


@dataclass
class A2AMessage:
    role: str = ""
    parts: List[A2APart] = field(default_factory=list)
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2AMessage":
        obj = _from_dict(cls, data)
        if data.get("parts"):
            obj.parts = [A2APart.from_dict(p) for p in data["parts"]]
        return obj


@dataclass
class A2AArtifact:
    id: str = ""
    parts: List[A2APart] = field(default_factory=list)
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2AArtifact":
        obj = _from_dict(cls, data)
        if data.get("parts"):
            obj.parts = [A2APart.from_dict(p) for p in data["parts"]]
        return obj


@dataclass
class A2ATaskStatus:
    state: str = ""
    message: str = ""
    timestamp: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2ATaskStatus":
        return _from_dict(cls, data)


@dataclass
class A2ATask:
    id: str = ""
    context_id: str = ""
    status: Optional[A2ATaskStatus] = None
    messages: List[A2AMessage] = field(default_factory=list)
    artifacts: List[A2AArtifact] = field(default_factory=list)
    metadata: Dict[str, Any] = field(default_factory=dict)
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2ATask":
        obj = _from_dict(cls, data)
        if data.get("status") and isinstance(data["status"], dict):
            obj.status = A2ATaskStatus.from_dict(data["status"])
        if data.get("messages"):
            obj.messages = [A2AMessage.from_dict(m) for m in data["messages"]]
        if data.get("artifacts"):
            obj.artifacts = [A2AArtifact.from_dict(a) for a in data["artifacts"]]
        return obj


@dataclass
class A2AAgentSkill:
    id: str = ""
    name: str = ""
    description: str = ""
    tags: List[str] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2AAgentSkill":
        return _from_dict(cls, data)


@dataclass
class WebhookTrigger:
    id: str = ""
    workspace_id: str = ""
    agent_id: str = ""
    name: str = ""
    token: str = ""
    token_prefix: str = ""
    is_active: bool = True
    has_secret: bool = False
    last_used_at: Optional[str] = None
    created_by_id: Optional[str] = None
    created_at: str = ""
    updated_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "WebhookTrigger":
        return _from_dict(cls, data)


@dataclass
class WebhookTriggerCreateResponse:
    id: str = ""
    workspace_id: str = ""
    agent_id: str = ""
    name: str = ""
    token: str = ""
    token_prefix: str = ""
    is_active: bool = True
    has_secret: bool = False
    last_used_at: Optional[str] = None
    created_by_id: Optional[str] = None
    created_at: str = ""
    updated_at: str = ""
    secret: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "WebhookTriggerCreateResponse":
        return _from_dict(cls, data)


# --- Media ---


@dataclass
class MediaModel:
    id: str = ""
    provider: str = ""
    model_id: str = ""
    display_name: str = ""
    media_type: str = ""
    is_active: bool = True
    capabilities: Dict[str, Any] = field(default_factory=dict)
    config_schema: Optional[Dict[str, Any]] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MediaModel":
        return _from_dict(cls, data)


@dataclass
class MediaGenerateResult:
    asset_id: str = ""
    url: str = ""
    media_type: str = ""
    provider: str = ""
    model: str = ""
    duration_ms: Optional[int] = None
    cost: float = 0.0
    metadata: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MediaGenerateResult":
        return _from_dict(cls, data)


@dataclass
class Asset:
    id: str = ""
    workspace_id: str = ""
    type: str = ""
    provider: str = ""
    model: str = ""
    file_name: str = ""
    file_size: Optional[int] = None
    mime_type: str = ""
    storage_path: str = ""
    execution_id: Optional[str] = None
    agent_id: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    created_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Asset":
        return _from_dict(cls, data)


@dataclass
class AssetSignedURL:
    url: str = ""
    expires_at: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AssetSignedURL":
        return _from_dict(cls, data)


@dataclass
class A2AAgentCapabilities:
    streaming: bool = False
    push_notifications: bool = False

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2AAgentCapabilities":
        return _from_dict(cls, data)


@dataclass
class A2AAgentCard:
    name: str = ""
    description: str = ""
    url: str = ""
    version: str = ""
    capabilities: Optional[A2AAgentCapabilities] = None
    skills: List[A2AAgentSkill] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "A2AAgentCard":
        obj = _from_dict(cls, data)
        if data.get("capabilities"):
            obj.capabilities = A2AAgentCapabilities.from_dict(data["capabilities"])
        if data.get("skills"):
            obj.skills = [A2AAgentSkill.from_dict(s) for s in data["skills"]]
        return obj



from .a2a import A2AResource, AsyncA2AResource
from .agents import AgentsResource, AsyncAgentsResource
from .approvals import ApprovalsResource, AsyncApprovalsResource
from .assets import AssetsResource, AsyncAssetsResource
from .chat import AsyncChatResource, ChatResource
from .costs import AsyncCostsResource, CostsResource
from .credentials import AsyncCredentialsResource, CredentialsResource
from .dashboard import AsyncDashboardResource, DashboardResource
from .data_sources import AsyncDataSourcesResource, DataSourcesResource
from .executions import AsyncExecutionsResource, ExecutionsResource
from .guardrails import AsyncGuardrailsResource, GuardrailsResource
from .llm_models import AsyncLLMModelsResource, LLMModelsResource
from .mcp_templates import AsyncMCPTemplatesResource, MCPTemplatesResource
from .mcp_tools import AsyncMCPToolsResource, MCPToolsResource
from .media import AsyncMediaResource, MediaResource
from .media_models import AsyncMediaModelsResource, MediaModelsResource
from .memories import AsyncMemoriesResource, MemoriesResource
from .prompts import AsyncPromptsResource, PromptsResource
from .scores import AsyncScoresResource, ScoresResource
from .sessions import AsyncSessionsResource, SessionsResource
from .templates import AsyncTemplatesResource, TemplatesResource
from .traces import AsyncTracesResource, TracesResource
from .webhook_triggers import AsyncWebhookTriggersResource, WebhookTriggersResource

__all__ = [
    "A2AResource",
    "AgentsResource",
    "ApprovalsResource",
    "AssetsResource",
    "AsyncA2AResource",
    "AsyncAgentsResource",
    "AsyncApprovalsResource",
    "AsyncAssetsResource",
    "AsyncChatResource",
    "AsyncCostsResource",
    "AsyncCredentialsResource",
    "AsyncDashboardResource",
    "AsyncDataSourcesResource",
    "AsyncExecutionsResource",
    "AsyncGuardrailsResource",
    "AsyncLLMModelsResource",
    "AsyncMCPTemplatesResource",
    "AsyncMCPToolsResource",
    "AsyncMediaModelsResource",
    "AsyncMediaResource",
    "AsyncMemoriesResource",
    "AsyncPromptsResource",
    "AsyncScoresResource",
    "AsyncSessionsResource",
    "AsyncTemplatesResource",
    "AsyncTracesResource",
    "AsyncWebhookTriggersResource",
    "ChatResource",
    "CostsResource",
    "CredentialsResource",
    "DashboardResource",
    "DataSourcesResource",
    "ExecutionsResource",
    "GuardrailsResource",
    "LLMModelsResource",
    "MCPTemplatesResource",
    "MCPToolsResource",
    "MediaModelsResource",
    "MediaResource",
    "MemoriesResource",
    "PromptsResource",
    "ScoresResource",
    "SessionsResource",
    "TemplatesResource",
    "TracesResource",
    "WebhookTriggersResource",
]



import httpx
import respx

from promptrails import PromptRails
from promptrails.types import Prompt

BASE = "http://localhost:8082"


def _client():
    return PromptRails(api_key="key", base_url=BASE, max_retries=0)


@respx.mock
def test_list_prompts():
    respx.get(f"{BASE}/api/v1/prompts").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": [
                    {
                        "id": "p1",
                        "name": "Prompt 1",
                        "status": "active",
                        "workspace_id": "ws",
                        "created_at": "2025-01-01T00:00:00Z",
                        "updated_at": "2025-01-01T00:00:00Z",
                    },
                ],
                "meta": {"total": 1, "page": 1, "limit": 20, "total_pages": 1},
            },
        )
    )
    with _client() as client:
        result = client.prompts.list()
    assert len(result.data) == 1
    assert isinstance(result.data[0], Prompt)
    assert result.data[0].name == "Prompt 1"


@respx.mock
def test_get_prompt():
    respx.get(f"{BASE}/api/v1/prompts/p1").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": {
                    "id": "p1",
                    "name": "Test Prompt",
                    "status": "active",
                    "workspace_id": "ws",
                    "created_at": "2025-01-01T00:00:00Z",
                    "updated_at": "2025-01-01T00:00:00Z",
                },
            },
        )
    )
    with _client() as client:
        prompt = client.prompts.get("p1")
    assert prompt.id == "p1"
    assert prompt.name == "Test Prompt"


@respx.mock
def test_create_prompt():
    respx.post(f"{BASE}/api/v1/prompts").mock(
        return_value=httpx.Response(
            201,
            json={
                "data": {
                    "id": "p2",
                    "name": "New Prompt",
                    "status": "draft",
                    "workspace_id": "ws",
                    "created_at": "2025-01-01T00:00:00Z",
                    "updated_at": "2025-01-01T00:00:00Z",
                },
            },
        )
    )
    with _client() as client:
        prompt = client.prompts.create(name="New Prompt")
    assert prompt.id == "p2"
    assert prompt.name == "New Prompt"


@respx.mock
def test_update_prompt():
    respx.patch(f"{BASE}/api/v1/prompts/p1").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": {
                    "id": "p1",
                    "name": "Updated",
                    "status": "active",
                    "workspace_id": "ws",
                    "created_at": "2025-01-01T00:00:00Z",
                    "updated_at": "2025-01-01T00:00:00Z",
                },
            },
        )
    )
    with _client() as client:
        prompt = client.prompts.update("p1", name="Updated")
    assert prompt.name == "Updated"


@respx.mock
def test_delete_prompt():
    respx.delete(f"{BASE}/api/v1/prompts/p1").mock(
        return_value=httpx.Response(200, json={"message": "deleted"})
    )
    with _client() as client:
        client.prompts.delete("p1")



from __future__ import annotations

from ._async_http import AsyncHTTPClient
from .config import Config
from .resources import (
    AsyncA2AResource,
    AsyncAgentsResource,
    AsyncApprovalsResource,
    AsyncAssetsResource,
    AsyncChatResource,
    AsyncCostsResource,
    AsyncCredentialsResource,
    AsyncDashboardResource,
    AsyncDataSourcesResource,
    AsyncExecutionsResource,
    AsyncGuardrailsResource,
    AsyncLLMModelsResource,
    AsyncMCPTemplatesResource,
    AsyncMCPToolsResource,
    AsyncMediaModelsResource,
    AsyncMediaResource,
    AsyncMemoriesResource,
    AsyncPromptsResource,
    AsyncScoresResource,
    AsyncSessionsResource,
    AsyncTemplatesResource,
    AsyncTracesResource,
    AsyncWebhookTriggersResource,
)


class AsyncPromptRails:
    """Async PromptRails API client."""

    def __init__(
        self,
        api_key: str,
        *,
        base_url: str = "https://api.promptrails.ai",
        timeout: float = 30.0,
        max_retries: int = 3,
    ):
        self._config = Config(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
        )
        self._http = AsyncHTTPClient(self._config)

        self.agents = AsyncAgentsResource(self._http)
        self.prompts = AsyncPromptsResource(self._http)
        self.executions = AsyncExecutionsResource(self._http)
        self.credentials = AsyncCredentialsResource(self._http)
        self.data_sources = AsyncDataSourcesResource(self._http)
        self.chat = AsyncChatResource(self._http)
        self.memories = AsyncMemoriesResource(self._http)
        self.traces = AsyncTracesResource(self._http)
        self.costs = AsyncCostsResource(self._http)
        self.mcp_tools = AsyncMCPToolsResource(self._http)
        self.mcp_templates = AsyncMCPTemplatesResource(self._http)
        self.guardrails = AsyncGuardrailsResource(self._http)
        self.llm_models = AsyncLLMModelsResource(self._http)
        self.approvals = AsyncApprovalsResource(self._http)
        self.templates = AsyncTemplatesResource(self._http)
        self.scores = AsyncScoresResource(self._http)
        self.dashboard = AsyncDashboardResource(self._http)
        self.sessions = AsyncSessionsResource(self._http)
        self.a2a = AsyncA2AResource(self._http)
        self.webhook_triggers = AsyncWebhookTriggersResource(self._http)
        self.media_models = AsyncMediaModelsResource(self._http)
        self.media = AsyncMediaResource(self._http)
        self.assets = AsyncAssetsResource(self._http)

    async def close(self):
        """Close the underlying HTTP client."""
        await self._http.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        await self.close()



from __future__ import annotations

import asyncio
from typing import Any, Dict, Optional

import httpx

from .config import Config
from .exceptions import RateLimitError, ServerError, raise_for_status


class AsyncHTTPClient:
    """Async HTTP wrapper around httpx."""

    def __init__(self, config: Config):
        self._config = config
        self._client = httpx.AsyncClient(
            base_url=config.base_url,
            headers=config.headers,
            timeout=config.timeout,
        )

    async def close(self):
        await self._client.aclose()

    async def request(
        self,
        method: str,
        path: str,
        *,
        json: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        for attempt in range(self._config.max_retries + 1):
            try:
                response = await self._client.request(method, path, json=json, params=params)
                body = response.json() if response.content else {}
                if response.status_code >= 500:
                    raise ServerError(
                        message=body.get("error", "Server error"),
                        status_code=response.status_code,
                    )
                raise_for_status(response.status_code, body)
                return body
            except (httpx.ConnectError, httpx.TimeoutException, ServerError):
                if attempt < self._config.max_retries:
                    await asyncio.sleep(min(2**attempt, 8))
                    continue
                raise
            except RateLimitError:
                raise

    async def get(self, path: str, **kwargs) -> Dict[str, Any]:
        return await self.request("GET", path, **kwargs)

    async def post(self, path: str, **kwargs) -> Dict[str, Any]:
        return await self.request("POST", path, **kwargs)

    async def patch(self, path: str, **kwargs) -> Dict[str, Any]:
        return await self.request("PATCH", path, **kwargs)

    async def put(self, path: str, **kwargs) -> Dict[str, Any]:
        return await self.request("PUT", path, **kwargs)

    async def delete(self, path: str, **kwargs) -> Dict[str, Any]:
        return await self.request("DELETE", path, **kwargs)



from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..pagination import PaginatedResponse
from ..types import Prompt, PromptVersion, RunPromptResponse
from .base import AsyncBaseResource, BaseResource


class PromptsResource(BaseResource):
    def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[Prompt]:
        body = self._http.get("/api/v1/prompts", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, Prompt.from_dict)

    def get(self, prompt_id: str) -> Prompt:
        body = self._http.get(f"/api/v1/prompts/{prompt_id}")
        return Prompt.from_dict(self._unwrap(body))

    def create(self, *, name: str, description: str = "") -> Prompt:
        body = self._http.post("/api/v1/prompts", json={"name": name, "description": description})
        return Prompt.from_dict(self._unwrap(body))

    def update(self, prompt_id: str, **kwargs) -> Prompt:
        body = self._http.patch(f"/api/v1/prompts/{prompt_id}", json=kwargs)
        return Prompt.from_dict(self._unwrap(body))

    def delete(self, prompt_id: str) -> None:
        self._http.delete(f"/api/v1/prompts/{prompt_id}")

    def list_versions(self, prompt_id: str) -> List[PromptVersion]:
        body = self._http.get(f"/api/v1/prompts/{prompt_id}/versions")
        data = self._unwrap(body)
        return [PromptVersion.from_dict(v) for v in (data if isinstance(data, list) else [])]

    def create_version(
        self,
        prompt_id: str,
        *,
        version: str,
        user_prompt: str,
        system_prompt: str = "",
        llm_model_id: Optional[str] = None,
        fallback_llm_model_id: Optional[str] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_p: Optional[float] = None,
        input_schema: Optional[Dict[str, Any]] = None,
        output_schema: Optional[Dict[str, Any]] = None,
        config: Optional[Dict[str, Any]] = None,
        set_current: bool = False,
        message: Optional[str] = None,
    ) -> PromptVersion:
        payload: Dict[str, Any] = {
            "version": version,
            "user_prompt": user_prompt,
            "system_prompt": system_prompt,
            "set_current": set_current,
        }
        if llm_model_id is not None:
            payload["llm_model_id"] = llm_model_id
        if fallback_llm_model_id is not None:
            payload["fallback_llm_model_id"] = fallback_llm_model_id
        if temperature is not None:
            payload["temperature"] = temperature
        if max_tokens is not None:
            payload["max_tokens"] = max_tokens
        if top_p is not None:
            payload["top_p"] = top_p
        if input_schema is not None:
            payload["input_schema"] = input_schema
        if output_schema is not None:
            payload["output_schema"] = output_schema
        if config is not None:
            payload["config"] = config
        if message is not None:
            payload["message"] = message
        body = self._http.post(f"/api/v1/prompts/{prompt_id}/versions", json=payload)
        return PromptVersion.from_dict(self._unwrap(body))

    def promote_version(self, prompt_id: str, version_id: str) -> Dict[str, Any]:
        body = self._http.put(f"/api/v1/prompts/{prompt_id}/versions/{version_id}/promote", json={})
        return self._unwrap(body)

    def preview(
        self,
        prompt_id: str,
        *,
        input: Dict[str, Any],
        version_id: Optional[str] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {"input": input}
        if version_id is not None:
            payload["version_id"] = version_id
        body = self._http.post(f"/api/v1/prompts/{prompt_id}/preview", json=payload)
        return self._unwrap(body)

    def run_prompt(self, prompt_id: str, data: Dict[str, Any]) -> RunPromptResponse:
        body = self._http.post(f"/api/v1/prompts/{prompt_id}/run", json=data)
        return RunPromptResponse.from_dict(self._unwrap(body))


class AsyncPromptsResource(AsyncBaseResource):
    async def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[Prompt]:
        body = await self._http.get("/api/v1/prompts", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, Prompt.from_dict)

    async def get(self, prompt_id: str) -> Prompt:
        body = await self._http.get(f"/api/v1/prompts/{prompt_id}")
        return Prompt.from_dict(self._unwrap(body))

    async def create(self, *, name: str, description: str = "") -> Prompt:
        body = await self._http.post(
            "/api/v1/prompts", json={"name": name, "description": description}
        )
        return Prompt.from_dict(self._unwrap(body))

    async def update(self, prompt_id: str, **kwargs) -> Prompt:
        body = await self._http.patch(f"/api/v1/prompts/{prompt_id}", json=kwargs)
        return Prompt.from_dict(self._unwrap(body))

    async def delete(self, prompt_id: str) -> None:
        await self._http.delete(f"/api/v1/prompts/{prompt_id}")

    async def list_versions(self, prompt_id: str) -> List[PromptVersion]:
        body = await self._http.get(f"/api/v1/prompts/{prompt_id}/versions")
        data = self._unwrap(body)
        return [PromptVersion.from_dict(v) for v in (data if isinstance(data, list) else [])]

    async def create_version(
        self,
        prompt_id: str,
        *,
        version: str,
        user_prompt: str,
        system_prompt: str = "",
        llm_model_id: Optional[str] = None,
        fallback_llm_model_id: Optional[str] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_p: Optional[float] = None,
        input_schema: Optional[Dict[str, Any]] = None,
        output_schema: Optional[Dict[str, Any]] = None,
        config: Optional[Dict[str, Any]] = None,
        set_current: bool = False,
        message: Optional[str] = None,
    ) -> PromptVersion:
        payload: Dict[str, Any] = {
            "version": version,
            "user_prompt": user_prompt,
            "system_prompt": system_prompt,
            "set_current": set_current,
        }
        if llm_model_id is not None:
            payload["llm_model_id"] = llm_model_id
        if fallback_llm_model_id is not None:
            payload["fallback_llm_model_id"] = fallback_llm_model_id
        if temperature is not None:
            payload["temperature"] = temperature
        if max_tokens is not None:
            payload["max_tokens"] = max_tokens
        if top_p is not None:
            payload["top_p"] = top_p
        if input_schema is not None:
            payload["input_schema"] = input_schema
        if output_schema is not None:
            payload["output_schema"] = output_schema
        if config is not None:
            payload["config"] = config
        if message is not None:
            payload["message"] = message
        body = await self._http.post(f"/api/v1/prompts/{prompt_id}/versions", json=payload)
        return PromptVersion.from_dict(self._unwrap(body))

    async def promote_version(self, prompt_id: str, version_id: str) -> Dict[str, Any]:
        body = await self._http.put(
            f"/api/v1/prompts/{prompt_id}/versions/{version_id}/promote", json={}
        )
        return self._unwrap(body)

    async def preview(
        self,
        prompt_id: str,
        *,
        input: Dict[str, Any],
        version_id: Optional[str] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {"input": input}
        if version_id is not None:
            payload["version_id"] = version_id
        body = await self._http.post(f"/api/v1/prompts/{prompt_id}/preview", json=payload)
        return self._unwrap(body)

    async def run_prompt(self, prompt_id: str, data: Dict[str, Any]) -> RunPromptResponse:
        body = await self._http.post(f"/api/v1/prompts/{prompt_id}/run", json=data)
        return RunPromptResponse.from_dict(self._unwrap(body))



from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..types import MediaGenerateResult
from .base import AsyncBaseResource, BaseResource


class MediaResource(BaseResource):
    """Resource for media generation (Media Studio)."""

    def generate(
        self,
        *,
        provider: str,
        media_type: str,
        model: str,
        prompt: Optional[str] = None,
        input_url: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        guardrails: Optional[List[Dict[str, Any]]] = None,
    ) -> MediaGenerateResult:
        """Generate media content.

        Args:
            provider: Media provider (e.g. ``fal``, ``elevenlabs``, ``stability``).
            media_type: Type of media to generate (``image``, ``speech``, ``video``).
            model: Model identifier to use.
            prompt: Text prompt for generation.
            input_url: Input URL for transformations (e.g. image-to-video).
            config: Provider-specific configuration options.
            guardrails: Optional guardrail configurations.

        Returns:
            Media generation result.
        """
        payload: Dict[str, Any] = {
            "provider": provider,
            "media_type": media_type,
            "model": model,
        }
        if prompt is not None:
            payload["prompt"] = prompt
        if input_url is not None:
            payload["input_url"] = input_url
        if config is not None:
            payload["config"] = config
        if guardrails is not None:
            payload["guardrails"] = guardrails
        body = self._http.post("/api/v1/media/generate", json=payload)
        return MediaGenerateResult.from_dict(self._unwrap(body))


class AsyncMediaResource(AsyncBaseResource):
    """Async resource for media generation (Media Studio)."""

    async def generate(
        self,
        *,
        provider: str,
        media_type: str,
        model: str,
        prompt: Optional[str] = None,
        input_url: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        guardrails: Optional[List[Dict[str, Any]]] = None,
    ) -> MediaGenerateResult:
        """Generate media content.

        Args:
            provider: Media provider (e.g. ``fal``, ``elevenlabs``, ``stability``).
            media_type: Type of media to generate (``image``, ``speech``, ``video``).
            model: Model identifier to use.
            prompt: Text prompt for generation.
            input_url: Input URL for transformations (e.g. image-to-video).
            config: Provider-specific configuration options.
            guardrails: Optional guardrail configurations.

        Returns:
            Media generation result.
        """
        payload: Dict[str, Any] = {
            "provider": provider,
            "media_type": media_type,
            "model": model,
        }
        if prompt is not None:
            payload["prompt"] = prompt
        if input_url is not None:
            payload["input_url"] = input_url
        if config is not None:
            payload["config"] = config
        if guardrails is not None:
            payload["guardrails"] = guardrails
        body = await self._http.post("/api/v1/media/generate", json=payload)
        return MediaGenerateResult.from_dict(self._unwrap(body))



from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..pagination import PaginatedResponse
from ..types import Score, ScoreAggregate, ScoreConfig
from .base import AsyncBaseResource, BaseResource


class ScoresResource(BaseResource):
    def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        trace_id: Optional[str] = None,
        name: Optional[str] = None,
        source: Optional[str] = None,
        data_type: Optional[str] = None,
        config_id: Optional[str] = None,
    ) -> PaginatedResponse[Score]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if trace_id:
            params["trace_id"] = trace_id
        if name:
            params["name"] = name
        if source:
            params["source"] = source
        if data_type:
            params["data_type"] = data_type
        if config_id:
            params["config_id"] = config_id
        body = self._http.get("/api/v1/scores", params=params)
        return PaginatedResponse.from_response(body, Score.from_dict)

    def get(self, score_id: str) -> Score:
        body = self._http.get(f"/api/v1/scores/{score_id}")
        return Score.from_dict(self._unwrap(body))

    def create(
        self,
        *,
        trace_id: str,
        name: str,
        value: Optional[float] = None,
        string_value: Optional[str] = None,
        data_type: str = "numeric",
        source: str = "api",
        span_id: Optional[str] = None,
        config_id: Optional[str] = None,
        comment: Optional[str] = None,
    ) -> Score:
        payload: Dict[str, Any] = {
            "trace_id": trace_id,
            "name": name,
            "data_type": data_type,
            "source": source,
        }
        if value is not None:
            payload["value"] = value
        if string_value is not None:
            payload["string_value"] = string_value
        if span_id:
            payload["span_id"] = span_id
        if config_id:
            payload["config_id"] = config_id
        if comment:
            payload["comment"] = comment
        body = self._http.post("/api/v1/scores", json=payload)
        return Score.from_dict(self._unwrap(body))

    def update(self, score_id: str, **kwargs: Any) -> Score:
        body = self._http.patch(f"/api/v1/scores/{score_id}", json=kwargs)
        return Score.from_dict(self._unwrap(body))

    def delete(self, score_id: str) -> None:
        self._http.delete(f"/api/v1/scores/{score_id}")

    def get_aggregates(
        self,
        *,
        trace_id: Optional[str] = None,
        name: Optional[str] = None,
    ) -> List[ScoreAggregate]:
        params: Dict[str, Any] = {}
        if trace_id:
            params["trace_id"] = trace_id
        if name:
            params["name"] = name
        body = self._http.get("/api/v1/scores/aggregates", params=params)
        data = self._unwrap(body)
        return [ScoreAggregate.from_dict(d) for d in (data if isinstance(data, list) else [])]

    # Score Configs
    def list_configs(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[ScoreConfig]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        body = self._http.get("/api/v1/score-configs", params=params)
        return PaginatedResponse.from_response(body, ScoreConfig.from_dict)

    def get_config(self, config_id: str) -> ScoreConfig:
        body = self._http.get(f"/api/v1/score-configs/{config_id}")
        return ScoreConfig.from_dict(self._unwrap(body))

    def create_config(
        self,
        *,
        name: str,
        data_type: str = "numeric",
        min_value: Optional[float] = None,
        max_value: Optional[float] = None,
        categories: Optional[List[Dict[str, Any]]] = None,
        description: Optional[str] = None,
    ) -> ScoreConfig:
        payload: Dict[str, Any] = {"name": name, "data_type": data_type}
        if min_value is not None:
            payload["min_value"] = min_value
        if max_value is not None:
            payload["max_value"] = max_value
        if categories:
            payload["categories"] = categories
        if description:
            payload["description"] = description
        body = self._http.post("/api/v1/score-configs", json=payload)
        return ScoreConfig.from_dict(self._unwrap(body))

    def update_config(self, config_id: str, **kwargs: Any) -> ScoreConfig:
        body = self._http.patch(f"/api/v1/score-configs/{config_id}", json=kwargs)
        return ScoreConfig.from_dict(self._unwrap(body))

    def delete_config(self, config_id: str) -> None:
        self._http.delete(f"/api/v1/score-configs/{config_id}")


class AsyncScoresResource(AsyncBaseResource):
    async def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        trace_id: Optional[str] = None,
        name: Optional[str] = None,
        source: Optional[str] = None,
        data_type: Optional[str] = None,
        config_id: Optional[str] = None,
    ) -> PaginatedResponse[Score]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if trace_id:
            params["trace_id"] = trace_id
        if name:
            params["name"] = name
        if source:
            params["source"] = source
        if data_type:
            params["data_type"] = data_type
        if config_id:
            params["config_id"] = config_id
        body = await self._http.get("/api/v1/scores", params=params)
        return PaginatedResponse.from_response(body, Score.from_dict)

    async def get(self, score_id: str) -> Score:
        body = await self._http.get(f"/api/v1/scores/{score_id}")
        return Score.from_dict(self._unwrap(body))

    async def create(
        self,
        *,
        trace_id: str,
        name: str,
        value: Optional[float] = None,
        string_value: Optional[str] = None,
        data_type: str = "numeric",
        source: str = "api",
        span_id: Optional[str] = None,
        config_id: Optional[str] = None,
        comment: Optional[str] = None,
    ) -> Score:
        payload: Dict[str, Any] = {
            "trace_id": trace_id,
            "name": name,
            "data_type": data_type,
            "source": source,
        }
        if value is not None:
            payload["value"] = value
        if string_value is not None:
            payload["string_value"] = string_value
        if span_id:
            payload["span_id"] = span_id
        if config_id:
            payload["config_id"] = config_id
        if comment:
            payload["comment"] = comment
        body = await self._http.post("/api/v1/scores", json=payload)
        return Score.from_dict(self._unwrap(body))

    async def update(self, score_id: str, **kwargs: Any) -> Score:
        body = await self._http.patch(f"/api/v1/scores/{score_id}", json=kwargs)
        return Score.from_dict(self._unwrap(body))

    async def delete(self, score_id: str) -> None:
        await self._http.delete(f"/api/v1/scores/{score_id}")

    async def get_aggregates(
        self,
        *,
        trace_id: Optional[str] = None,
        name: Optional[str] = None,
    ) -> List[ScoreAggregate]:
        params: Dict[str, Any] = {}
        if trace_id:
            params["trace_id"] = trace_id
        if name:
            params["name"] = name
        body = await self._http.get("/api/v1/scores/aggregates", params=params)
        data = self._unwrap(body)
        return [ScoreAggregate.from_dict(d) for d in (data if isinstance(data, list) else [])]

    async def list_configs(
        self, *, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[ScoreConfig]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        body = await self._http.get("/api/v1/score-configs", params=params)
        return PaginatedResponse.from_response(body, ScoreConfig.from_dict)

    async def get_config(self, config_id: str) -> ScoreConfig:
        body = await self._http.get(f"/api/v1/score-configs/{config_id}")
        return ScoreConfig.from_dict(self._unwrap(body))

    async def create_config(
        self,
        *,
        name: str,
        data_type: str = "numeric",
        min_value: Optional[float] = None,
        max_value: Optional[float] = None,
        categories: Optional[List[Dict[str, Any]]] = None,
        description: Optional[str] = None,
    ) -> ScoreConfig:
        payload: Dict[str, Any] = {"name": name, "data_type": data_type}
        if min_value is not None:
            payload["min_value"] = min_value
        if max_value is not None:
            payload["max_value"] = max_value
        if categories:
            payload["categories"] = categories
        if description:
            payload["description"] = description
        body = await self._http.post("/api/v1/score-configs", json=payload)
        return ScoreConfig.from_dict(self._unwrap(body))

    async def update_config(self, config_id: str, **kwargs: Any) -> ScoreConfig:
        body = await self._http.patch(f"/api/v1/score-configs/{config_id}", json=kwargs)
        return ScoreConfig.from_dict(self._unwrap(body))

    async def delete_config(self, config_id: str) -> None:
        await self._http.delete(f"/api/v1/score-configs/{config_id}")



from __future__ import annotations

from ..types import Guardrail
from .base import AsyncBaseResource, BaseResource


class GuardrailsResource(BaseResource):
    def update(self, guardrail_id: str, **kwargs) -> Guardrail:
        body = self._http.patch(f"/api/v1/guardrails/{guardrail_id}", json=kwargs)
        return Guardrail.from_dict(self._unwrap(body))

    def delete(self, guardrail_id: str) -> None:
        self._http.delete(f"/api/v1/guardrails/{guardrail_id}")


class AsyncGuardrailsResource(AsyncBaseResource):
    async def update(self, guardrail_id: str, **kwargs) -> Guardrail:
        body = await self._http.patch(f"/api/v1/guardrails/{guardrail_id}", json=kwargs)
        return Guardrail.from_dict(self._unwrap(body))

    async def delete(self, guardrail_id: str) -> None:
        await self._http.delete(f"/api/v1/guardrails/{guardrail_id}")



import httpx
import respx

from promptrails import PromptRails

BASE = "http://localhost:8082"


def _client():
    return PromptRails(api_key="key", base_url=BASE, max_retries=0)


@respx.mock
def test_list_executions():
    respx.get(f"{BASE}/api/v1/executions").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": [
                    {
                        "id": "e1",
                        "agent_id": "a1",
                        "status": "completed",
                        "workspace_id": "ws",
                        "created_at": "2025-01-01T00:00:00Z",
                        "updated_at": "2025-01-01T00:00:00Z",
                    },
                ],
                "meta": {"total": 1, "page": 1, "limit": 20, "total_pages": 1},
            },
        )
    )
    with _client() as client:
        result = client.executions.list()
    assert len(result.data) == 1
    assert result.data[0].status == "completed"


@respx.mock
def test_get_execution():
    respx.get(f"{BASE}/api/v1/executions/e1").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": {
                    "id": "e1",
                    "agent_id": "a1",
                    "status": "completed",
                    "workspace_id": "ws",
                    "input": {"query": "test"},
                    "output": {"result": "ok"},
                    "created_at": "2025-01-01T00:00:00Z",
                    "updated_at": "2025-01-01T00:00:00Z",
                },
            },
        )
    )
    with _client() as client:
        execution = client.executions.get("e1")
    assert execution.id == "e1"
    assert execution.status == "completed"



"""PromptRails Python SDK."""

from .async_client import AsyncPromptRails
from .client import PromptRails
from .exceptions import (
    ForbiddenError,
    NotFoundError,
    PromptRailsError,
    RateLimitError,
    ServerError,
    UnauthorizedError,
    ValidationError,
)
from .pagination import PaginatedResponse, PaginationMeta
from .types import (
    Agent,
    AgentExecution,
    AgentMemory,
    AgentVersion,
    ApprovalRequest,
    Asset,
    AssetSignedURL,
    AvailableModelEntry,
    AvailableModelGroup,
    ChatMessage,
    ChatSession,
    CostSummary,
    Credential,
    DataSource,
    DataSourceVersion,
    ExecutionResult,
    FlowTemplate,
    Guardrail,
    InstallMCPTemplateRequest,
    LLMModel,
    MCPCallToolResult,
    MCPDiscoveredTool,
    MCPDiscoverResult,
    MCPTemplate,
    MCPTemplateParameterSpec,
    MCPTool,
    MCPToolCallContent,
    MediaGenerateResult,
    MediaModel,
    Prompt,
    PromptVersion,
    RunPromptResponse,
    Trace,
)

__version__ = "0.1.0"

__all__ = [
    "Agent",
    "AgentExecution",
    "AgentMemory",
    "AgentVersion",
    "ApprovalRequest",
    "Asset",
    "AssetSignedURL",
    "AsyncPromptRails",
    "AvailableModelEntry",
    "AvailableModelGroup",
    "ChatMessage",
    "ChatSession",
    "CostSummary",
    "Credential",
    "DataSource",
    "DataSourceVersion",
    "ExecutionResult",
    "FlowTemplate",
    "ForbiddenError",
    "Guardrail",
    "InstallMCPTemplateRequest",
    "LLMModel",
    "MCPCallToolResult",
    "MCPDiscoverResult",
    "MCPDiscoveredTool",
    "MCPTemplate",
    "MCPTemplateParameterSpec",
    "MCPTool",
    "MCPToolCallContent",
    "MediaGenerateResult",
    "MediaModel",
    "NotFoundError",
    "PaginatedResponse",
    "PaginationMeta",
    "Prompt",
    "PromptRails",
    "PromptRailsError",
    "PromptVersion",
    "RateLimitError",
    "RunPromptResponse",
    "ServerError",
    "Trace",
    "UnauthorizedError",
    "ValidationError",
]



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import FlowTemplate
from .base import AsyncBaseResource, BaseResource


class TemplatesResource(BaseResource):
    """Flow template operations."""

    def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        status: Optional[str] = None,
        category: Optional[str] = None,
    ) -> PaginatedResponse[FlowTemplate]:
        """List flow templates."""
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if status:
            params["status"] = status
        if category:
            params["category"] = category
        body = self._http.get("/api/v1/templates", params=params)
        return PaginatedResponse.from_response(body, FlowTemplate.from_dict)

    def get(self, template_id: str) -> FlowTemplate:
        """Get a flow template by ID."""
        body = self._http.get(f"/api/v1/templates/{template_id}")
        return FlowTemplate.from_dict(self._unwrap(body))


class AsyncTemplatesResource(AsyncBaseResource):
    """Async flow template operations."""

    async def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        status: Optional[str] = None,
        category: Optional[str] = None,
    ) -> PaginatedResponse[FlowTemplate]:
        """List flow templates."""
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if status:
            params["status"] = status
        if category:
            params["category"] = category
        body = await self._http.get("/api/v1/templates", params=params)
        return PaginatedResponse.from_response(body, FlowTemplate.from_dict)

    async def get(self, template_id: str) -> FlowTemplate:
        """Get a flow template by ID."""
        body = await self._http.get(f"/api/v1/templates/{template_id}")
        return FlowTemplate.from_dict(self._unwrap(body))



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import AgentExecution
from .base import AsyncBaseResource, BaseResource


class ExecutionsResource(BaseResource):
    def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        agent_id: Optional[str] = None,
        session_id: Optional[str] = None,
        status: Optional[str] = None,
    ) -> PaginatedResponse[AgentExecution]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if agent_id:
            params["agent_id"] = agent_id
        if session_id:
            params["session_id"] = session_id
        if status:
            params["status"] = status
        body = self._http.get("/api/v1/executions", params=params)
        return PaginatedResponse.from_response(body, AgentExecution.from_dict)

    def get(self, execution_id: str) -> AgentExecution:
        body = self._http.get(f"/api/v1/executions/{execution_id}")
        return AgentExecution.from_dict(self._unwrap(body))


class AsyncExecutionsResource(AsyncBaseResource):
    async def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        agent_id: Optional[str] = None,
        session_id: Optional[str] = None,
        status: Optional[str] = None,
    ) -> PaginatedResponse[AgentExecution]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if agent_id:
            params["agent_id"] = agent_id
        if session_id:
            params["session_id"] = session_id
        if status:
            params["status"] = status
        body = await self._http.get("/api/v1/executions", params=params)
        return PaginatedResponse.from_response(body, AgentExecution.from_dict)

    async def get(self, execution_id: str) -> AgentExecution:
        body = await self._http.get(f"/api/v1/executions/{execution_id}")
        return AgentExecution.from_dict(self._unwrap(body))



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict, Generic, List, TypeVar

T = TypeVar("T")


@dataclass
class PaginationMeta:
    total: int
    page: int
    limit: int
    pages: int


@dataclass
class PaginatedResponse(Generic[T]):
    data: List[T]
    meta: PaginationMeta

    @classmethod
    def from_response(cls, body: Dict[str, Any], item_factory) -> "PaginatedResponse":
        raw_data = body.get("data", [])
        items = [item_factory(item) for item in raw_data]
        meta_raw = body.get("meta", {})
        meta = PaginationMeta(
            total=meta_raw.get("total", 0),
            page=meta_raw.get("page", 1),
            limit=meta_raw.get("limit", 20),
            pages=meta_raw.get("total_pages", 0),
        )
        return cls(data=items, meta=meta)



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import WebhookTrigger, WebhookTriggerCreateResponse
from .base import AsyncBaseResource, BaseResource


class WebhookTriggersResource(BaseResource):
    def list(
        self, *, agent_id: Optional[str] = None, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[WebhookTrigger]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if agent_id:
            params["agent_id"] = agent_id
        body = self._http.get("/api/v1/webhook-triggers", params=params)
        return PaginatedResponse.from_response(body, WebhookTrigger.from_dict)

    def get(self, trigger_id: str) -> WebhookTrigger:
        body = self._http.get(f"/api/v1/webhook-triggers/{trigger_id}")
        return WebhookTrigger.from_dict(self._unwrap(body))

    def create(
        self,
        *,
        name: str,
        agent_id: str,
        generate_secret: bool = False,
    ) -> WebhookTriggerCreateResponse:
        payload: Dict[str, Any] = {
            "name": name,
            "agent_id": agent_id,
            "generate_secret": generate_secret,
        }
        body = self._http.post("/api/v1/webhook-triggers", json=payload)
        return WebhookTriggerCreateResponse.from_dict(self._unwrap(body))

    def update(self, trigger_id: str, **kwargs: Any) -> WebhookTrigger:
        body = self._http.patch(f"/api/v1/webhook-triggers/{trigger_id}", json=kwargs)
        return WebhookTrigger.from_dict(self._unwrap(body))

    def delete(self, trigger_id: str) -> None:
        self._http.delete(f"/api/v1/webhook-triggers/{trigger_id}")


class AsyncWebhookTriggersResource(AsyncBaseResource):
    async def list(
        self, *, agent_id: Optional[str] = None, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[WebhookTrigger]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if agent_id:
            params["agent_id"] = agent_id
        body = await self._http.get("/api/v1/webhook-triggers", params=params)
        return PaginatedResponse.from_response(body, WebhookTrigger.from_dict)

    async def get(self, trigger_id: str) -> WebhookTrigger:
        body = await self._http.get(f"/api/v1/webhook-triggers/{trigger_id}")
        return WebhookTrigger.from_dict(self._unwrap(body))

    async def create(
        self,
        *,
        name: str,
        agent_id: str,
        generate_secret: bool = False,
    ) -> WebhookTriggerCreateResponse:
        payload: Dict[str, Any] = {
            "name": name,
            "agent_id": agent_id,
            "generate_secret": generate_secret,
        }
        body = await self._http.post("/api/v1/webhook-triggers", json=payload)
        return WebhookTriggerCreateResponse.from_dict(self._unwrap(body))

    async def update(self, trigger_id: str, **kwargs: Any) -> WebhookTrigger:
        body = await self._http.patch(f"/api/v1/webhook-triggers/{trigger_id}", json=kwargs)
        return WebhookTrigger.from_dict(self._unwrap(body))

    async def delete(self, trigger_id: str) -> None:
        await self._http.delete(f"/api/v1/webhook-triggers/{trigger_id}")



from __future__ import annotations

from typing import Any, Dict, Optional


class PromptRailsError(Exception):
    """Base exception for all PromptRails SDK errors."""

    def __init__(
        self,
        message: str,
        status_code: Optional[int] = None,
        code: Optional[str] = None,
        details: Optional[Dict[str, Any]] = None,
    ):
        super().__init__(message)
        self.message = message
        self.status_code = status_code
        self.code = code
        self.details = details or {}


class ValidationError(PromptRailsError):
    """Raised on 400 Bad Request."""

    pass


class UnauthorizedError(PromptRailsError):
    """Raised on 401 Unauthorized."""

    pass


class ForbiddenError(PromptRailsError):
    """Raised on 403 Forbidden."""

    pass


class NotFoundError(PromptRailsError):
    """Raised on 404 Not Found."""

    pass


class RateLimitError(PromptRailsError):
    """Raised on 429 Too Many Requests."""

    pass


class ServerError(PromptRailsError):
    """Raised on 5xx Server Error."""

    pass


_STATUS_MAP = {
    400: ValidationError,
    401: UnauthorizedError,
    403: ForbiddenError,
    404: NotFoundError,
    429: RateLimitError,
}


def raise_for_status(status_code: int, body: Dict[str, Any]) -> None:
    if 200 <= status_code < 300:
        return
    error_data = body.get("error", {})
    if isinstance(error_data, str):
        message = error_data
        code = None
        details = {}
    else:
        message = error_data.get("message", body.get("message", "Unknown error"))
        code = error_data.get("code")
        details = error_data.get("details", {})
    exc_cls = _STATUS_MAP.get(status_code, ServerError)
    raise exc_cls(
        message=message,
        status_code=status_code,
        code=code,
        details=details,
    )



from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..pagination import PaginatedResponse
from ..types import Trace
from .base import AsyncBaseResource, BaseResource


class TracesResource(BaseResource):
    def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        trace_id: Optional[str] = None,
        kind: Optional[str] = None,
    ) -> PaginatedResponse[Trace]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if trace_id:
            params["trace_id"] = trace_id
        if kind:
            params["kind"] = kind
        body = self._http.get("/api/v1/traces", params=params)
        return PaginatedResponse.from_response(body, Trace.from_dict)

    def get_by_trace_id(self, trace_id: str) -> List[Trace]:
        body = self._http.get(f"/api/v1/traces/{trace_id}")
        data = self._unwrap(body)
        return [
            Trace.from_dict(t) for t in (data if isinstance(data, list) else [data] if data else [])
        ]


class AsyncTracesResource(AsyncBaseResource):
    async def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        trace_id: Optional[str] = None,
        kind: Optional[str] = None,
    ) -> PaginatedResponse[Trace]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if trace_id:
            params["trace_id"] = trace_id
        if kind:
            params["kind"] = kind
        body = await self._http.get("/api/v1/traces", params=params)
        return PaginatedResponse.from_response(body, Trace.from_dict)

    async def get_by_trace_id(self, trace_id: str) -> List[Trace]:
        body = await self._http.get(f"/api/v1/traces/{trace_id}")
        data = self._unwrap(body)
        return [
            Trace.from_dict(t) for t in (data if isinstance(data, list) else [data] if data else [])
        ]



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import A2AAgentCard, A2ATask
from .base import AsyncBaseResource, BaseResource


class A2AResource(BaseResource):
    def get_agent_card(self, agent_id: str) -> A2AAgentCard:
        body = self._http.get(f"/a2a/agents/{agent_id}/agent-card.json")
        return A2AAgentCard.from_dict(body)

    def send_message(
        self,
        agent_id: str,
        message: str,
        *,
        blocking: bool = True,
        task_id: Optional[str] = None,
        context_id: Optional[str] = None,
    ) -> A2ATask:
        params: Dict[str, Any] = {
            "message": {
                "role": "user",
                "parts": [{"type": "text", "text": message}],
            },
        }
        if task_id:
            params["task_id"] = task_id
        if context_id:
            params["context_id"] = context_id
        if not blocking:
            params.setdefault("configuration", {})
            params["configuration"]["blocking"] = False

        rpc_body: Dict[str, Any] = {
            "jsonrpc": "2.0",
            "method": "message/send",
            "params": params,
            "id": 1,
        }
        body = self._http.post(f"/a2a/agents/{agent_id}", json=rpc_body)
        result = body.get("result", body)
        return A2ATask.from_dict(result)

    def get_task(self, task_id: str) -> A2ATask:
        rpc_body: Dict[str, Any] = {
            "jsonrpc": "2.0",
            "method": "tasks/get",
            "params": {"id": task_id},
            "id": 1,
        }
        # We need the agent_id for the URL but tasks/get only needs task_id in params.
        # The A2A endpoint expects POST to /a2a/agents/{agentId} but for task operations
        # we use a convenience wrapper that the server routes appropriately.
        body = self._http.post("/a2a/tasks/get", json=rpc_body)
        result = body.get("result", body)
        return A2ATask.from_dict(result)

    def list_tasks(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        context_id: Optional[str] = None,
        status: Optional[str] = None,
    ) -> PaginatedResponse[A2ATask]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if context_id:
            params["context_id"] = context_id
        if status:
            params["status"] = status
        body = self._http.get("/api/v1/a2a/tasks", params=params)
        return PaginatedResponse.from_response(body, A2ATask.from_dict)

    def cancel_task(self, task_id: str) -> A2ATask:
        rpc_body: Dict[str, Any] = {
            "jsonrpc": "2.0",
            "method": "tasks/cancel",
            "params": {"id": task_id},
            "id": 1,
        }
        body = self._http.post("/a2a/tasks/cancel", json=rpc_body)
        result = body.get("result", body)
        return A2ATask.from_dict(result)


class AsyncA2AResource(AsyncBaseResource):
    async def get_agent_card(self, agent_id: str) -> A2AAgentCard:
        body = await self._http.get(f"/a2a/agents/{agent_id}/agent-card.json")
        return A2AAgentCard.from_dict(body)

    async def send_message(
        self,
        agent_id: str,
        message: str,
        *,
        blocking: bool = True,
        task_id: Optional[str] = None,
        context_id: Optional[str] = None,
    ) -> A2ATask:
        params: Dict[str, Any] = {
            "message": {
                "role": "user",
                "parts": [{"type": "text", "text": message}],
            },
        }
        if task_id:
            params["task_id"] = task_id
        if context_id:
            params["context_id"] = context_id
        if not blocking:
            params.setdefault("configuration", {})
            params["configuration"]["blocking"] = False

        rpc_body: Dict[str, Any] = {
            "jsonrpc": "2.0",
            "method": "message/send",
            "params": params,
            "id": 1,
        }
        body = await self._http.post(f"/a2a/agents/{agent_id}", json=rpc_body)
        result = body.get("result", body)
        return A2ATask.from_dict(result)

    async def get_task(self, task_id: str) -> A2ATask:
        rpc_body: Dict[str, Any] = {
            "jsonrpc": "2.0",
            "method": "tasks/get",
            "params": {"id": task_id},
            "id": 1,
        }
        body = await self._http.post("/a2a/tasks/get", json=rpc_body)
        result = body.get("result", body)
        return A2ATask.from_dict(result)

    async def list_tasks(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        context_id: Optional[str] = None,
        status: Optional[str] = None,
    ) -> PaginatedResponse[A2ATask]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if context_id:
            params["context_id"] = context_id
        if status:
            params["status"] = status
        body = await self._http.get("/api/v1/a2a/tasks", params=params)
        return PaginatedResponse.from_response(body, A2ATask.from_dict)

    async def cancel_task(self, task_id: str) -> A2ATask:
        rpc_body: Dict[str, Any] = {
            "jsonrpc": "2.0",
            "method": "tasks/cancel",
            "params": {"id": task_id},
            "id": 1,
        }
        body = await self._http.post("/a2a/tasks/cancel", json=rpc_body)
        result = body.get("result", body)
        return A2ATask.from_dict(result)



from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..types import MediaModel
from .base import AsyncBaseResource, BaseResource


class MediaModelsResource(BaseResource):
    """Resource for listing available media models."""

    def list(
        self,
        *,
        provider: Optional[str] = None,
        media_type: Optional[str] = None,
        is_active: Optional[bool] = None,
    ) -> List[MediaModel]:
        """List available media models.

        Args:
            provider: Filter by provider (e.g. ``fal``, ``elevenlabs``).
            media_type: Filter by media type (e.g. ``image``, ``speech``, ``video``).
            is_active: Filter by active status.

        Returns:
            List of media models.
        """
        params: Dict[str, Any] = {}
        if provider is not None:
            params["provider"] = provider
        if media_type is not None:
            params["media_type"] = media_type
        if is_active is not None:
            params["is_active"] = str(is_active).lower()
        body = self._http.get("/api/v1/media-models", params=params)
        data = self._unwrap(body)
        return [MediaModel.from_dict(m) for m in (data if isinstance(data, list) else [])]


class AsyncMediaModelsResource(AsyncBaseResource):
    """Async resource for listing available media models."""

    async def list(
        self,
        *,
        provider: Optional[str] = None,
        media_type: Optional[str] = None,
        is_active: Optional[bool] = None,
    ) -> List[MediaModel]:
        """List available media models.

        Args:
            provider: Filter by provider (e.g. ``fal``, ``elevenlabs``).
            media_type: Filter by media type (e.g. ``image``, ``speech``, ``video``).
            is_active: Filter by active status.

        Returns:
            List of media models.
        """
        params: Dict[str, Any] = {}
        if provider is not None:
            params["provider"] = provider
        if media_type is not None:
            params["media_type"] = media_type
        if is_active is not None:
            params["is_active"] = str(is_active).lower()
        body = await self._http.get("/api/v1/media-models", params=params)
        data = self._unwrap(body)
        return [MediaModel.from_dict(m) for m in (data if isinstance(data, list) else [])]



import pytest

from promptrails import AsyncPromptRails, PromptRails
from promptrails.config import Config
from promptrails.exceptions import (
    NotFoundError,
    RateLimitError,
    UnauthorizedError,
    ValidationError,
    raise_for_status,
)


class TestConfig:
    def test_defaults(self):
        cfg = Config(api_key="key")
        assert cfg.base_url == "https://api.promptrails.ai"
        assert cfg.timeout == 30.0
        assert cfg.max_retries == 3

    def test_trailing_slash_stripped(self):
        cfg = Config(api_key="key", base_url="http://localhost:8082/")
        assert cfg.base_url == "http://localhost:8082"

    def test_headers(self):
        cfg = Config(api_key="test-key")
        h = cfg.headers
        assert h["X-API-Key"] == "test-key"
        assert "X-Workspace-ID" not in h
        assert h["Content-Type"] == "application/json"


class TestClient:
    def test_creates_resources(self):
        client = PromptRails(api_key="k")
        assert client.agents is not None
        assert client.prompts is not None
        assert client.executions is not None
        assert client.credentials is not None
        assert client.data_sources is not None
        assert client.chat is not None
        assert client.memories is not None
        assert client.traces is not None
        assert client.costs is not None
        assert client.mcp_tools is not None
        assert client.guardrails is not None
        assert client.approvals is not None
        client.close()

    def test_context_manager(self):
        with PromptRails(api_key="k") as client:
            assert client.agents is not None


class TestAsyncClient:
    def test_creates_resources(self):
        client = AsyncPromptRails(api_key="k")
        assert client.agents is not None
        assert client.prompts is not None
        assert client.executions is not None


class TestExceptions:
    def test_raise_400(self):
        with pytest.raises(ValidationError) as exc_info:
            raise_for_status(400, {"error": {"message": "bad input", "code": "VALIDATION"}})
        assert exc_info.value.status_code == 400
        assert exc_info.value.code == "VALIDATION"

    def test_raise_401(self):
        with pytest.raises(UnauthorizedError):
            raise_for_status(401, {"error": "unauthorized"})

    def test_raise_404(self):
        with pytest.raises(NotFoundError):
            raise_for_status(404, {"error": {"message": "not found"}})

    def test_raise_429(self):
        with pytest.raises(RateLimitError):
            raise_for_status(429, {"error": {"message": "rate limited"}})

    def test_no_raise_on_200(self):
        raise_for_status(200, {"data": {}})



from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..pagination import PaginatedResponse
from ..types import AvailableModelGroup, LLMModel
from .base import AsyncBaseResource, BaseResource


class LLMModelsResource(BaseResource):
    def list(
        self, *, page: int = 1, limit: int = 20, provider: Optional[str] = None
    ) -> PaginatedResponse[LLMModel]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if provider is not None:
            params["provider"] = provider
        body = self._http.get("/api/v1/llm-models", params=params)
        return PaginatedResponse.from_response(body, LLMModel.from_dict)

    def list_available(self) -> List[AvailableModelGroup]:
        body = self._http.get("/api/v1/llm-models/available")
        data = self._unwrap(body)
        return [AvailableModelGroup.from_dict(g) for g in (data if isinstance(data, list) else [])]


class AsyncLLMModelsResource(AsyncBaseResource):
    async def list(
        self, *, page: int = 1, limit: int = 20, provider: Optional[str] = None
    ) -> PaginatedResponse[LLMModel]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if provider is not None:
            params["provider"] = provider
        body = await self._http.get("/api/v1/llm-models", params=params)
        return PaginatedResponse.from_response(body, LLMModel.from_dict)

    async def list_available(self) -> List[AvailableModelGroup]:
        body = await self._http.get("/api/v1/llm-models/available")
        data = self._unwrap(body)
        return [AvailableModelGroup.from_dict(g) for g in (data if isinstance(data, list) else [])]



from __future__ import annotations

from typing import Any, Dict

from ..pagination import PaginatedResponse
from ..types import SessionSummary
from .base import AsyncBaseResource, BaseResource


class SessionsResource(BaseResource):
    def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[SessionSummary]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        body = self._http.get("/api/v1/sessions", params=params)
        return PaginatedResponse.from_response(body, SessionSummary.from_dict)

    def get(self, session_id: str) -> SessionSummary:
        body = self._http.get(f"/api/v1/sessions/{session_id}")
        return SessionSummary.from_dict(self._unwrap(body))


class AsyncSessionsResource(AsyncBaseResource):
    async def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[SessionSummary]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        body = await self._http.get("/api/v1/sessions", params=params)
        return PaginatedResponse.from_response(body, SessionSummary.from_dict)

    async def get(self, session_id: str) -> SessionSummary:
        body = await self._http.get(f"/api/v1/sessions/{session_id}")
        return SessionSummary.from_dict(self._unwrap(body))



from __future__ import annotations

from ..types import AgentMemory
from .base import AsyncBaseResource, BaseResource


class MemoriesResource(BaseResource):
    def get(self, memory_id: str) -> AgentMemory:
        body = self._http.get(f"/api/v1/memories/{memory_id}")
        return AgentMemory.from_dict(self._unwrap(body))

    def delete(self, memory_id: str) -> None:
        self._http.delete(f"/api/v1/memories/{memory_id}")


class AsyncMemoriesResource(AsyncBaseResource):
    async def get(self, memory_id: str) -> AgentMemory:
        body = await self._http.get(f"/api/v1/memories/{memory_id}")
        return AgentMemory.from_dict(self._unwrap(body))

    async def delete(self, memory_id: str) -> None:
        await self._http.delete(f"/api/v1/memories/{memory_id}")



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import ChatMessage, ChatSession
from .base import AsyncBaseResource, BaseResource


class ChatResource(BaseResource):
    def list_sessions(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[ChatSession]:
        body = self._http.get("/api/v1/chat/sessions", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, ChatSession.from_dict)

    def get_session(self, session_id: str) -> ChatSession:
        body = self._http.get(f"/api/v1/chat/sessions/{session_id}")
        return ChatSession.from_dict(self._unwrap(body))

    def create_session(
        self, *, agent_id: str, title: str = "", metadata: Optional[Dict[str, Any]] = None
    ) -> ChatSession:
        payload: Dict[str, Any] = {"agent_id": agent_id}
        if title:
            payload["title"] = title
        if metadata is not None:
            payload["metadata"] = metadata
        body = self._http.post("/api/v1/chat/sessions", json=payload)
        return ChatSession.from_dict(self._unwrap(body))

    def delete_session(self, session_id: str) -> None:
        self._http.delete(f"/api/v1/chat/sessions/{session_id}")

    def list_messages(
        self, session_id: str, *, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[ChatMessage]:
        body = self._http.get(
            f"/api/v1/chat/sessions/{session_id}/messages", params={"page": page, "limit": limit}
        )
        return PaginatedResponse.from_response(body, ChatMessage.from_dict)

    def send_message(self, session_id: str, *, content: str) -> ChatMessage:
        body = self._http.post(
            f"/api/v1/chat/sessions/{session_id}/messages", json={"content": content}
        )
        return ChatMessage.from_dict(self._unwrap(body))


class AsyncChatResource(AsyncBaseResource):
    async def list_sessions(
        self, *, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[ChatSession]:
        body = await self._http.get("/api/v1/chat/sessions", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, ChatSession.from_dict)

    async def get_session(self, session_id: str) -> ChatSession:
        body = await self._http.get(f"/api/v1/chat/sessions/{session_id}")
        return ChatSession.from_dict(self._unwrap(body))

    async def create_session(
        self, *, agent_id: str, title: str = "", metadata: Optional[Dict[str, Any]] = None
    ) -> ChatSession:
        payload: Dict[str, Any] = {"agent_id": agent_id}
        if title:
            payload["title"] = title
        if metadata is not None:
            payload["metadata"] = metadata
        body = await self._http.post("/api/v1/chat/sessions", json=payload)
        return ChatSession.from_dict(self._unwrap(body))

    async def delete_session(self, session_id: str) -> None:
        await self._http.delete(f"/api/v1/chat/sessions/{session_id}")

    async def list_messages(
        self, session_id: str, *, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[ChatMessage]:
        body = await self._http.get(
            f"/api/v1/chat/sessions/{session_id}/messages", params={"page": page, "limit": limit}
        )
        return PaginatedResponse.from_response(body, ChatMessage.from_dict)

    async def send_message(self, session_id: str, *, content: str) -> ChatMessage:
        body = await self._http.post(
            f"/api/v1/chat/sessions/{session_id}/messages", json={"content": content}
        )
        return ChatMessage.from_dict(self._unwrap(body))



from __future__ import annotations

from typing import Any, Dict, List, Optional

from ..pagination import PaginatedResponse
from ..types import Agent, AgentMemory, AgentVersion, ExecutionResult, Guardrail
from .base import AsyncBaseResource, BaseResource


class AgentsResource(BaseResource):
    def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        type: Optional[str] = None,
        name: Optional[str] = None,
    ) -> PaginatedResponse[Agent]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if type:
            params["type"] = type
        if name:
            params["name"] = name
        body = self._http.get("/api/v1/agents", params=params)
        return PaginatedResponse.from_response(body, Agent.from_dict)

    def get(self, agent_id: str) -> Agent:
        body = self._http.get(f"/api/v1/agents/{agent_id}")
        return Agent.from_dict(self._unwrap(body))

    def create(
        self,
        *,
        name: str,
        type: str,
        description: str = "",
        template_id: Optional[str] = None,
    ) -> Agent:
        payload: Dict[str, Any] = {"name": name, "type": type, "description": description}
        if template_id is not None:
            payload["template_id"] = template_id
        body = self._http.post("/api/v1/agents", json=payload)
        return Agent.from_dict(self._unwrap(body))

    def update(self, agent_id: str, **kwargs) -> Agent:
        body = self._http.patch(f"/api/v1/agents/{agent_id}", json=kwargs)
        return Agent.from_dict(self._unwrap(body))

    def delete(self, agent_id: str) -> None:
        self._http.delete(f"/api/v1/agents/{agent_id}")

    def execute(
        self,
        agent_id: str,
        *,
        input: Dict[str, Any],
        session_id: Optional[str] = None,
        version_id: Optional[str] = None,
        sync: Optional[bool] = None,
    ) -> ExecutionResult:
        payload: Dict[str, Any] = {"input": input}
        if session_id:
            payload["session_id"] = session_id
        if version_id is not None:
            payload["version_id"] = version_id
        if sync is not None:
            payload["sync"] = sync
        body = self._http.post(f"/api/v1/agents/{agent_id}/execute", json=payload)
        return ExecutionResult.from_dict(self._unwrap(body))

    def list_versions(self, agent_id: str) -> List[AgentVersion]:
        body = self._http.get(f"/api/v1/agents/{agent_id}/versions")
        data = self._unwrap(body)
        if isinstance(data, list):
            return [AgentVersion.from_dict(v) for v in data]
        return [AgentVersion.from_dict(v) for v in data] if data else []

    def create_version(
        self,
        agent_id: str,
        *,
        version: str,
        config: Optional[Dict[str, Any]] = None,
        input_schema: Optional[Dict[str, Any]] = None,
        output_schema: Optional[Dict[str, Any]] = None,
        set_current: bool = False,
        message: Optional[str] = None,
    ) -> AgentVersion:
        payload: Dict[str, Any] = {"version": version, "set_current": set_current}
        if config is not None:
            payload["config"] = config
        if input_schema is not None:
            payload["input_schema"] = input_schema
        if output_schema is not None:
            payload["output_schema"] = output_schema
        if message is not None:
            payload["message"] = message
        body = self._http.post(f"/api/v1/agents/{agent_id}/versions", json=payload)
        return AgentVersion.from_dict(self._unwrap(body))

    def promote_version(self, agent_id: str, version_id: str) -> Dict[str, Any]:
        body = self._http.put(f"/api/v1/agents/{agent_id}/versions/{version_id}/promote", json={})
        return self._unwrap(body)

    def preview(
        self,
        agent_id: str,
        *,
        input: Dict[str, Any],
        version_id: Optional[str] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {"input": input}
        if version_id is not None:
            payload["version_id"] = version_id
        body = self._http.post(f"/api/v1/agents/{agent_id}/preview", json=payload)
        return self._unwrap(body)

    def list_guardrails(self, agent_id: str) -> List[Guardrail]:
        body = self._http.get(f"/api/v1/agents/{agent_id}/guardrails")
        data = self._unwrap(body)
        return [Guardrail.from_dict(g) for g in (data if isinstance(data, list) else [])]

    def create_guardrail(
        self,
        agent_id: str,
        *,
        type: str,
        scanner_type: str,
        action: str = "block",
        config: Optional[Dict[str, Any]] = None,
    ) -> Guardrail:
        payload: Dict[str, Any] = {"type": type, "scanner_type": scanner_type, "action": action}
        if config is not None:
            payload["config"] = config
        body = self._http.post(f"/api/v1/agents/{agent_id}/guardrails", json=payload)
        return Guardrail.from_dict(self._unwrap(body))

    def list_memories(
        self, agent_id: str, *, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[AgentMemory]:
        body = self._http.get(
            f"/api/v1/agents/{agent_id}/memories", params={"page": page, "limit": limit}
        )
        return PaginatedResponse.from_response(body, AgentMemory.from_dict)

    def create_memory(
        self,
        agent_id: str,
        *,
        content: str,
        memory_type: str,
        importance: Optional[float] = None,
        metadata: Optional[Dict[str, Any]] = None,
        chat_session_id: Optional[str] = None,
    ) -> AgentMemory:
        payload: Dict[str, Any] = {"content": content, "memory_type": memory_type}
        if importance is not None:
            payload["importance"] = importance
        if metadata is not None:
            payload["metadata"] = metadata
        if chat_session_id is not None:
            payload["chat_session_id"] = chat_session_id
        body = self._http.post(f"/api/v1/agents/{agent_id}/memories", json=payload)
        return AgentMemory.from_dict(self._unwrap(body))

    def search_memories(
        self,
        agent_id: str,
        *,
        query: str,
        threshold: Optional[float] = None,
        limit: Optional[int] = None,
    ) -> List[AgentMemory]:
        payload: Dict[str, Any] = {"query": query}
        if threshold is not None:
            payload["threshold"] = threshold
        if limit is not None:
            payload["limit"] = limit
        body = self._http.post(f"/api/v1/agents/{agent_id}/memories/search", json=payload)
        data = self._unwrap(body)
        return [AgentMemory.from_dict(m) for m in (data if isinstance(data, list) else [])]

    def delete_all_memories(self, agent_id: str) -> None:
        self._http.delete(f"/api/v1/agents/{agent_id}/memories")


class AsyncAgentsResource(AsyncBaseResource):
    async def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        type: Optional[str] = None,
        name: Optional[str] = None,
    ) -> PaginatedResponse[Agent]:
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if type:
            params["type"] = type
        if name:
            params["name"] = name
        body = await self._http.get("/api/v1/agents", params=params)
        return PaginatedResponse.from_response(body, Agent.from_dict)

    async def get(self, agent_id: str) -> Agent:
        body = await self._http.get(f"/api/v1/agents/{agent_id}")
        return Agent.from_dict(self._unwrap(body))

    async def create(
        self,
        *,
        name: str,
        type: str,
        description: str = "",
        template_id: Optional[str] = None,
    ) -> Agent:
        payload: Dict[str, Any] = {"name": name, "type": type, "description": description}
        if template_id is not None:
            payload["template_id"] = template_id
        body = await self._http.post("/api/v1/agents", json=payload)
        return Agent.from_dict(self._unwrap(body))

    async def update(self, agent_id: str, **kwargs) -> Agent:
        body = await self._http.patch(f"/api/v1/agents/{agent_id}", json=kwargs)
        return Agent.from_dict(self._unwrap(body))

    async def delete(self, agent_id: str) -> None:
        await self._http.delete(f"/api/v1/agents/{agent_id}")

    async def execute(
        self,
        agent_id: str,
        *,
        input: Dict[str, Any],
        session_id: Optional[str] = None,
        version_id: Optional[str] = None,
        sync: Optional[bool] = None,
    ) -> ExecutionResult:
        payload: Dict[str, Any] = {"input": input}
        if session_id:
            payload["session_id"] = session_id
        if version_id is not None:
            payload["version_id"] = version_id
        if sync is not None:
            payload["sync"] = sync
        body = await self._http.post(f"/api/v1/agents/{agent_id}/execute", json=payload)
        return ExecutionResult.from_dict(self._unwrap(body))

    async def list_versions(self, agent_id: str) -> List[AgentVersion]:
        body = await self._http.get(f"/api/v1/agents/{agent_id}/versions")
        data = self._unwrap(body)
        return [AgentVersion.from_dict(v) for v in (data if isinstance(data, list) else [])]

    async def create_version(
        self,
        agent_id: str,
        *,
        version: str,
        config: Optional[Dict[str, Any]] = None,
        input_schema: Optional[Dict[str, Any]] = None,
        output_schema: Optional[Dict[str, Any]] = None,
        set_current: bool = False,
        message: Optional[str] = None,
    ) -> AgentVersion:
        payload: Dict[str, Any] = {"version": version, "set_current": set_current}
        if config is not None:
            payload["config"] = config
        if input_schema is not None:
            payload["input_schema"] = input_schema
        if output_schema is not None:
            payload["output_schema"] = output_schema
        if message is not None:
            payload["message"] = message
        body = await self._http.post(f"/api/v1/agents/{agent_id}/versions", json=payload)
        return AgentVersion.from_dict(self._unwrap(body))

    async def promote_version(self, agent_id: str, version_id: str) -> Dict[str, Any]:
        body = await self._http.put(
            f"/api/v1/agents/{agent_id}/versions/{version_id}/promote", json={}
        )
        return self._unwrap(body)

    async def preview(
        self,
        agent_id: str,
        *,
        input: Dict[str, Any],
        version_id: Optional[str] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {"input": input}
        if version_id is not None:
            payload["version_id"] = version_id
        body = await self._http.post(f"/api/v1/agents/{agent_id}/preview", json=payload)
        return self._unwrap(body)

    async def list_guardrails(self, agent_id: str) -> List[Guardrail]:
        body = await self._http.get(f"/api/v1/agents/{agent_id}/guardrails")
        data = self._unwrap(body)
        return [Guardrail.from_dict(g) for g in (data if isinstance(data, list) else [])]

    async def create_guardrail(
        self,
        agent_id: str,
        *,
        type: str,
        scanner_type: str,
        action: str = "block",
        config: Optional[Dict[str, Any]] = None,
    ) -> Guardrail:
        payload: Dict[str, Any] = {"type": type, "scanner_type": scanner_type, "action": action}
        if config is not None:
            payload["config"] = config
        body = await self._http.post(f"/api/v1/agents/{agent_id}/guardrails", json=payload)
        return Guardrail.from_dict(self._unwrap(body))

    async def list_memories(
        self, agent_id: str, *, page: int = 1, limit: int = 20
    ) -> PaginatedResponse[AgentMemory]:
        body = await self._http.get(
            f"/api/v1/agents/{agent_id}/memories", params={"page": page, "limit": limit}
        )
        return PaginatedResponse.from_response(body, AgentMemory.from_dict)

    async def create_memory(
        self,
        agent_id: str,
        *,
        content: str,
        memory_type: str,
        importance: Optional[float] = None,
        metadata: Optional[Dict[str, Any]] = None,
        chat_session_id: Optional[str] = None,
    ) -> AgentMemory:
        payload: Dict[str, Any] = {"content": content, "memory_type": memory_type}
        if importance is not None:
            payload["importance"] = importance
        if metadata is not None:
            payload["metadata"] = metadata
        if chat_session_id is not None:
            payload["chat_session_id"] = chat_session_id
        body = await self._http.post(f"/api/v1/agents/{agent_id}/memories", json=payload)
        return AgentMemory.from_dict(self._unwrap(body))

    async def search_memories(
        self,
        agent_id: str,
        *,
        query: str,
        threshold: Optional[float] = None,
        limit: Optional[int] = None,
    ) -> List[AgentMemory]:
        payload: Dict[str, Any] = {"query": query}
        if threshold is not None:
            payload["threshold"] = threshold
        if limit is not None:
            payload["limit"] = limit
        body = await self._http.post(f"/api/v1/agents/{agent_id}/memories/search", json=payload)
        data = self._unwrap(body)
        return [AgentMemory.from_dict(m) for m in (data if isinstance(data, list) else [])]

    async def delete_all_memories(self, agent_id: str) -> None:
        await self._http.delete(f"/api/v1/agents/{agent_id}/memories")



from __future__ import annotations

from typing import Literal, Optional

from ..pagination import PaginatedResponse
from ..types import ApprovalRequest
from .base import AsyncBaseResource, BaseResource


class ApprovalsResource(BaseResource):
    def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        status: Optional[str] = None,
    ) -> PaginatedResponse[ApprovalRequest]:
        params: dict = {"page": page, "limit": limit}
        if status:
            params["status"] = status
        body = self._http.get("/api/v1/approvals", params=params)
        return PaginatedResponse.from_response(body, ApprovalRequest.from_dict)

    def get(self, approval_id: str) -> ApprovalRequest:
        body = self._http.get(f"/api/v1/approvals/{approval_id}")
        return ApprovalRequest.from_dict(self._unwrap(body))

    def decide(
        self,
        approval_id: str,
        *,
        decision: Literal["approved", "rejected"],
        reason: Optional[str] = None,
    ) -> ApprovalRequest:
        payload: dict = {"decision": decision}
        if reason is not None:
            payload["reason"] = reason
        body = self._http.post(f"/api/v1/approvals/{approval_id}/decide", json=payload)
        return ApprovalRequest.from_dict(self._unwrap(body))


class AsyncApprovalsResource(AsyncBaseResource):
    async def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        status: Optional[str] = None,
    ) -> PaginatedResponse[ApprovalRequest]:
        params: dict = {"page": page, "limit": limit}
        if status:
            params["status"] = status
        body = await self._http.get("/api/v1/approvals", params=params)
        return PaginatedResponse.from_response(body, ApprovalRequest.from_dict)

    async def get(self, approval_id: str) -> ApprovalRequest:
        body = await self._http.get(f"/api/v1/approvals/{approval_id}")
        return ApprovalRequest.from_dict(self._unwrap(body))

    async def decide(
        self,
        approval_id: str,
        *,
        decision: Literal["approved", "rejected"],
        reason: Optional[str] = None,
    ) -> ApprovalRequest:
        payload: dict = {"decision": decision}
        if reason is not None:
            payload["reason"] = reason
        body = await self._http.post(f"/api/v1/approvals/{approval_id}/decide", json=payload)
        return ApprovalRequest.from_dict(self._unwrap(body))



from __future__ import annotations

from typing import Any, Dict

from ..pagination import PaginatedResponse
from ..types import Credential
from .base import AsyncBaseResource, BaseResource


class CredentialsResource(BaseResource):
    def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[Credential]:
        body = self._http.get("/api/v1/credentials", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, Credential.from_dict)

    def get(self, credential_id: str) -> Credential:
        body = self._http.get(f"/api/v1/credentials/{credential_id}")
        return Credential.from_dict(self._unwrap(body))

    def create(
        self,
        *,
        name: str,
        type: str,
        category: str,
        value: str,
        description: str = "",
        is_default: bool = False,
        schema_type: str = "",
    ) -> Credential:
        payload = {
            "name": name,
            "type": type,
            "category": category,
            "value": value,
            "description": description,
            "is_default": is_default,
        }
        if schema_type:
            payload["schema_type"] = schema_type
        body = self._http.post("/api/v1/credentials", json=payload)
        return Credential.from_dict(self._unwrap(body))

    def update(self, credential_id: str, **kwargs) -> Credential:
        body = self._http.patch(f"/api/v1/credentials/{credential_id}", json=kwargs)
        return Credential.from_dict(self._unwrap(body))

    def delete(self, credential_id: str) -> None:
        self._http.delete(f"/api/v1/credentials/{credential_id}")

    def set_default(self, credential_id: str) -> Credential:
        body = self._http.post(f"/api/v1/credentials/{credential_id}/set-default")
        return Credential.from_dict(self._unwrap(body))

    def check_connection(self, credential_id: str) -> Dict[str, Any]:
        body = self._http.post(f"/api/v1/credentials/{credential_id}/check")
        return self._unwrap(body)


class AsyncCredentialsResource(AsyncBaseResource):
    async def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[Credential]:
        body = await self._http.get("/api/v1/credentials", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, Credential.from_dict)

    async def get(self, credential_id: str) -> Credential:
        body = await self._http.get(f"/api/v1/credentials/{credential_id}")
        return Credential.from_dict(self._unwrap(body))

    async def create(
        self,
        *,
        name: str,
        type: str,
        category: str,
        value: str,
        description: str = "",
        is_default: bool = False,
        schema_type: str = "",
    ) -> Credential:
        payload = {
            "name": name,
            "type": type,
            "category": category,
            "value": value,
            "description": description,
            "is_default": is_default,
        }
        if schema_type:
            payload["schema_type"] = schema_type
        body = await self._http.post("/api/v1/credentials", json=payload)
        return Credential.from_dict(self._unwrap(body))

    async def update(self, credential_id: str, **kwargs) -> Credential:
        body = await self._http.patch(f"/api/v1/credentials/{credential_id}", json=kwargs)
        return Credential.from_dict(self._unwrap(body))

    async def delete(self, credential_id: str) -> None:
        await self._http.delete(f"/api/v1/credentials/{credential_id}")

    async def set_default(self, credential_id: str) -> Credential:
        body = await self._http.post(f"/api/v1/credentials/{credential_id}/set-default")
        return Credential.from_dict(self._unwrap(body))

    async def check_connection(self, credential_id: str) -> Dict[str, Any]:
        body = await self._http.post(f"/api/v1/credentials/{credential_id}/check")
        return self._unwrap(body)



from __future__ import annotations

import time
from typing import Any, Dict, Optional

import httpx

from .config import Config
from .exceptions import RateLimitError, ServerError, raise_for_status


class HTTPClient:
    """Synchronous HTTP wrapper around httpx."""

    def __init__(self, config: Config):
        self._config = config
        self._client = httpx.Client(
            base_url=config.base_url,
            headers=config.headers,
            timeout=config.timeout,
        )

    def close(self):
        self._client.close()

    def request(
        self,
        method: str,
        path: str,
        *,
        json: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        for attempt in range(self._config.max_retries + 1):
            try:
                response = self._client.request(method, path, json=json, params=params)
                body = response.json() if response.content else {}
                if response.status_code >= 500:
                    raise ServerError(
                        message=body.get("error", "Server error"),
                        status_code=response.status_code,
                    )
                raise_for_status(response.status_code, body)
                return body
            except (httpx.ConnectError, httpx.TimeoutException, ServerError):
                if attempt < self._config.max_retries:
                    time.sleep(min(2**attempt, 8))
                    continue
                raise
            except RateLimitError:
                raise

    def get(self, path: str, **kwargs) -> Dict[str, Any]:
        return self.request("GET", path, **kwargs)

    def post(self, path: str, **kwargs) -> Dict[str, Any]:
        return self.request("POST", path, **kwargs)

    def patch(self, path: str, **kwargs) -> Dict[str, Any]:
        return self.request("PATCH", path, **kwargs)

    def put(self, path: str, **kwargs) -> Dict[str, Any]:
        return self.request("PUT", path, **kwargs)

    def delete(self, path: str, **kwargs) -> Dict[str, Any]:
        return self.request("DELETE", path, **kwargs)



from __future__ import annotations

from typing import Any, Dict

from .._async_http import AsyncHTTPClient
from .._http import HTTPClient


class BaseResource:
    """Base class for sync resource access."""

    def __init__(self, http: HTTPClient):
        self._http = http

    def _unwrap(self, body: Dict[str, Any]) -> Any:
        return body.get("data", body)


class AsyncBaseResource:
    """Base class for async resource access."""

    def __init__(self, http: AsyncHTTPClient):
        self._http = http

    def _unwrap(self, body: Dict[str, Any]) -> Any:
        return body.get("data", body)



from dataclasses import dataclass


@dataclass
class Config:
    api_key: str
    base_url: str = "https://api.promptrails.ai"
    timeout: float = 30.0
    max_retries: int = 3

    def __post_init__(self):
        self.base_url = self.base_url.rstrip("/")

    @property
    def headers(self) -> dict:
        return {
            "X-API-Key": self.api_key,
            "Content-Type": "application/json",
            "Accept": "application/json",
        }



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import MCPTemplate, MCPTool
from .base import AsyncBaseResource, BaseResource


class MCPTemplatesResource(BaseResource):
    """MCP template marketplace operations."""

    def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        category: Optional[str] = None,
        type: Optional[str] = None,
        search: Optional[str] = None,
        is_active: Optional[bool] = None,
    ) -> PaginatedResponse[MCPTemplate]:
        """List MCP templates."""
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if category:
            params["category"] = category
        if type:
            params["type"] = type
        if search:
            params["search"] = search
        if is_active is not None:
            params["is_active"] = is_active
        body = self._http.get("/api/v1/mcp-templates", params=params)
        return PaginatedResponse.from_response(body, MCPTemplate.from_dict)

    def get(self, template_id: str) -> MCPTemplate:
        """Get an MCP template by ID."""
        body = self._http.get(f"/api/v1/mcp-templates/{template_id}")
        return MCPTemplate.from_dict(self._unwrap(body))

    def get_by_slug(self, slug: str) -> MCPTemplate:
        """Get an MCP template by slug."""
        body = self._http.get(f"/api/v1/mcp-templates/slug/{slug}")
        return MCPTemplate.from_dict(self._unwrap(body))

    def install(
        self,
        slug: str,
        *,
        name: str,
        parameters: Optional[Dict[str, str]] = None,
    ) -> MCPTool:
        """Install an MCP template, creating an MCP tool."""
        payload: Dict[str, Any] = {"name": name}
        if parameters is not None:
            payload["parameters"] = parameters
        body = self._http.post(f"/api/v1/mcp-templates/slug/{slug}/install", json=payload)
        return MCPTool.from_dict(self._unwrap(body))


class AsyncMCPTemplatesResource(AsyncBaseResource):
    """Async MCP template marketplace operations."""

    async def list(
        self,
        *,
        page: int = 1,
        limit: int = 20,
        category: Optional[str] = None,
        type: Optional[str] = None,
        search: Optional[str] = None,
        is_active: Optional[bool] = None,
    ) -> PaginatedResponse[MCPTemplate]:
        """List MCP templates."""
        params: Dict[str, Any] = {"page": page, "limit": limit}
        if category:
            params["category"] = category
        if type:
            params["type"] = type
        if search:
            params["search"] = search
        if is_active is not None:
            params["is_active"] = is_active
        body = await self._http.get("/api/v1/mcp-templates", params=params)
        return PaginatedResponse.from_response(body, MCPTemplate.from_dict)

    async def get(self, template_id: str) -> MCPTemplate:
        """Get an MCP template by ID."""
        body = await self._http.get(f"/api/v1/mcp-templates/{template_id}")
        return MCPTemplate.from_dict(self._unwrap(body))

    async def get_by_slug(self, slug: str) -> MCPTemplate:
        """Get an MCP template by slug."""
        body = await self._http.get(f"/api/v1/mcp-templates/slug/{slug}")
        return MCPTemplate.from_dict(self._unwrap(body))

    async def install(
        self,
        slug: str,
        *,
        name: str,
        parameters: Optional[Dict[str, str]] = None,
    ) -> MCPTool:
        """Install an MCP template, creating an MCP tool."""
        payload: Dict[str, Any] = {"name": name}
        if parameters is not None:
            payload["parameters"] = parameters
        body = await self._http.post(f"/api/v1/mcp-templates/slug/{slug}/install", json=payload)
        return MCPTool.from_dict(self._unwrap(body))



import httpx
import pytest
import respx

from promptrails import PromptRails
from promptrails.exceptions import NotFoundError
from promptrails.types import Agent

BASE = "http://localhost:8082"


def _client():
    return PromptRails(api_key="key", base_url=BASE, max_retries=0)


@respx.mock
def test_list_agents():
    respx.get(f"{BASE}/api/v1/agents").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": [
                    {
                        "id": "a1",
                        "name": "Agent 1",
                        "type": "simple",
                        "status": "active",
                        "workspace_id": "ws",
                        "created_at": "2025-01-01T00:00:00Z",
                        "updated_at": "2025-01-01T00:00:00Z",
                    },
                    {
                        "id": "a2",
                        "name": "Agent 2",
                        "type": "chain",
                        "status": "draft",
                        "workspace_id": "ws",
                        "created_at": "2025-01-01T00:00:00Z",
                        "updated_at": "2025-01-01T00:00:00Z",
                    },
                ],
                "meta": {"total": 2, "page": 1, "limit": 20, "total_pages": 1},
            },
        )
    )
    with _client() as client:
        result = client.agents.list()
    assert len(result.data) == 2
    assert result.meta.total == 2
    assert isinstance(result.data[0], Agent)
    assert result.data[0].name == "Agent 1"


@respx.mock
def test_get_agent():
    respx.get(f"{BASE}/api/v1/agents/a1").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": {
                    "id": "a1",
                    "name": "Test Agent",
                    "type": "simple",
                    "status": "active",
                    "workspace_id": "ws",
                    "created_at": "2025-01-01T00:00:00Z",
                    "updated_at": "2025-01-01T00:00:00Z",
                },
            },
        )
    )
    with _client() as client:
        agent = client.agents.get("a1")
    assert agent.id == "a1"
    assert agent.name == "Test Agent"


@respx.mock
def test_create_agent():
    respx.post(f"{BASE}/api/v1/agents").mock(
        return_value=httpx.Response(
            201,
            json={
                "data": {
                    "id": "a3",
                    "name": "New Agent",
                    "type": "simple",
                    "status": "draft",
                    "workspace_id": "ws",
                    "created_at": "2025-01-01T00:00:00Z",
                    "updated_at": "2025-01-01T00:00:00Z",
                },
            },
        )
    )
    with _client() as client:
        agent = client.agents.create(name="New Agent", type="simple")
    assert agent.id == "a3"
    assert agent.name == "New Agent"


@respx.mock
def test_update_agent():
    respx.patch(f"{BASE}/api/v1/agents/a1").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": {
                    "id": "a1",
                    "name": "Updated",
                    "type": "simple",
                    "status": "active",
                    "workspace_id": "ws",
                    "created_at": "2025-01-01T00:00:00Z",
                    "updated_at": "2025-01-01T00:00:00Z",
                },
            },
        )
    )
    with _client() as client:
        agent = client.agents.update("a1", name="Updated")
    assert agent.name == "Updated"


@respx.mock
def test_delete_agent():
    respx.delete(f"{BASE}/api/v1/agents/a1").mock(
        return_value=httpx.Response(200, json={"message": "deleted"})
    )
    with _client() as client:
        client.agents.delete("a1")


@respx.mock
def test_execute_agent():
    respx.post(f"{BASE}/api/v1/agents/a1/execute").mock(
        return_value=httpx.Response(
            200,
            json={
                "data": {
                    "output": {"result": "hello"},
                    "trace_id": "t1",
                    "execution_id": "e1",
                    "status": "completed",
                    "cost": 0.001,
                },
            },
        )
    )
    with _client() as client:
        result = client.agents.execute("a1", input={"prompt": "hi"})
    assert result.output == {"result": "hello"}
    assert result.trace_id == "t1"


@respx.mock
def test_not_found():
    respx.get(f"{BASE}/api/v1/agents/missing").mock(
        return_value=httpx.Response(404, json={"error": {"message": "agent not found"}})
    )
    with _client() as client:
        with pytest.raises(NotFoundError):
            client.agents.get("missing")



from __future__ import annotations

from typing import Any, Dict, Optional

from ..pagination import PaginatedResponse
from ..types import MCPCallToolResult, MCPDiscoverResult, MCPTool
from .base import AsyncBaseResource, BaseResource


class MCPToolsResource(BaseResource):
    def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[MCPTool]:
        body = self._http.get("/api/v1/mcp-tools", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, MCPTool.from_dict)

    def get(self, tool_id: str) -> MCPTool:
        body = self._http.get(f"/api/v1/mcp-tools/{tool_id}")
        return MCPTool.from_dict(self._unwrap(body))

    def create(
        self,
        *,
        name: str,
        type: str,
        config: Optional[Dict[str, Any]] = None,
        schema: Optional[Dict[str, Any]] = None,
        credential_id: Optional[str] = None,
        template_id: Optional[str] = None,
        status: Optional[str] = None,
    ) -> MCPTool:
        payload: Dict[str, Any] = {"name": name, "type": type}
        if config is not None:
            payload["config"] = config
        if schema is not None:
            payload["schema"] = schema
        if credential_id is not None:
            payload["credential_id"] = credential_id
        if template_id is not None:
            payload["template_id"] = template_id
        if status is not None:
            payload["status"] = status
        body = self._http.post("/api/v1/mcp-tools", json=payload)
        return MCPTool.from_dict(self._unwrap(body))

    def update(self, tool_id: str, **kwargs) -> MCPTool:
        body = self._http.patch(f"/api/v1/mcp-tools/{tool_id}", json=kwargs)
        return MCPTool.from_dict(self._unwrap(body))

    def delete(self, tool_id: str) -> None:
        self._http.delete(f"/api/v1/mcp-tools/{tool_id}")

    def discover_tools(self, tool_id: str) -> MCPDiscoverResult:
        """Discover available sub-tools for an MCP tool."""
        body = self._http.post(f"/api/v1/mcp-tools/{tool_id}/discover")
        return MCPDiscoverResult.from_dict(self._unwrap(body))

    def call_tool(
        self,
        tool_id: str,
        *,
        tool_name: str,
        arguments: Optional[Dict[str, Any]] = None,
    ) -> MCPCallToolResult:
        """Execute a tool call with the provided arguments."""
        payload: Dict[str, Any] = {"tool_name": tool_name}
        if arguments is not None:
            payload["arguments"] = arguments
        body = self._http.post(f"/api/v1/mcp-tools/{tool_id}/call", json=payload)
        return MCPCallToolResult.from_dict(self._unwrap(body))


class AsyncMCPToolsResource(AsyncBaseResource):
    async def list(self, *, page: int = 1, limit: int = 20) -> PaginatedResponse[MCPTool]:
        body = await self._http.get("/api/v1/mcp-tools", params={"page": page, "limit": limit})
        return PaginatedResponse.from_response(body, MCPTool.from_dict)

    async def get(self, tool_id: str) -> MCPTool:
        body = await self._http.get(f"/api/v1/mcp-tools/{tool_id}")
        return MCPTool.from_dict(self._unwrap(body))

    async def create(
        self,
        *,
        name: str,
        type: str,
        config: Optional[Dict[str, Any]] = None,
        schema: Optional[Dict[str, Any]] = None,
        credential_id: Optional[str] = None,
        template_id: Optional[str] = None,
        status: Optional[str] = None,
    ) -> MCPTool:
        payload: Dict[str, Any] = {"name": name, "type": type}
        if config is not None:
            payload["config"] = config
        if schema is not None:
            payload["schema"] = schema
        if credential_id is not None:
            payload["credential_id"] = credential_id
        if template_id is not None:
            payload["template_id"] = template_id
        if status is not None:
            payload["status"] = status
        body = await self._http.post("/api/v1/mcp-tools", json=payload)
        return MCPTool.from_dict(self._unwrap(body))

    async def update(self, tool_id: str, **kwargs) -> MCPTool:
        body = await self._http.patch(f"/api/v1/mcp-tools/{tool_id}", json=kwargs)
        return MCPTool.from_dict(self._unwrap(body))

    async def delete(self, tool_id: str) -> None:
        await self._http.delete(f"/api/v1/mcp-tools/{tool_id}")

    async def discover_tools(self, tool_id: str) -> MCPDiscoverResult:
        """Discover available sub-tools for an MCP tool."""
        body = await self._http.post(f"/api/v1/mcp-tools/{tool_id}/discover")
        return MCPDiscoverResult.from_dict(self._unwrap(body))

    async def call_tool(
        self,
        tool_id: str,
        *,
        tool_name: str,
        arguments: Optional[Dict[str, Any]] = None,
    ) -> MCPCallToolResult:
        """Execute a tool call with the provided arguments."""
        payload: Dict[str, Any] = {"tool_name": tool_name}
        if arguments is not None:
            payload["arguments"] = arguments
        body = await self._http.post(f"/api/v1/mcp-tools/{tool_id}/call", json=payload)
        return MCPCallToolResult.from_dict(self._unwrap(body))



from __future__ import annotations

from typing import Any, Dict, Optional

from ..types import DashboardMetrics
from .base import AsyncBaseResource, BaseResource


class DashboardResource(BaseResource):
    def get_metrics(self, *, days: Optional[int] = None) -> DashboardMetrics:
        params: Dict[str, Any] = {}
        if days is not None:
            params["days"] = days
        body = self._http.get("/api/v1/dashboard/metrics", params=params)
        return DashboardMetrics.from_dict(self._unwrap(body))


class AsyncDashboardResource(AsyncBaseResource):
    async def get_metrics(self, *, days: Optional[int] = None) -> DashboardMetrics:
        params: Dict[str, Any] = {}
        if days is not None:
            params["days"] = days
        body = await self._http.get("/api/v1/dashboard/metrics", params=params)
        return DashboardMetrics.from_dict(self._unwrap(body))
